"""Models for the chat app."""

from uuid import uuid4

from django.core.exceptions import ValidationError
from django.db.models import Q, Max
from django.db.models.signals import post_save
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from aparnik.contrib.filefields.models import FileField
from django.db import models
from django.contrib.auth import get_user_model
from django_enumfield import enum

User = get_user_model()


class ChatSessionTypeEnum(enum.Enum):
    PRIVATE = 0
    GROUP = 1
    CHANNEL = 2

    labels = {
        PRIVATE: _('PRIVATE'),
        GROUP: _('GROUP'),
        CHANNEL: _('CHANNEL')
    }


class TrackableDateModel(models.Model):
    """Abstract model to Track the creation/updated date for a model."""

    created_at = models.DateTimeField(auto_now_add=True)
    update_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True

    @property
    def resourcetype(self):
        return self._meta.object_name


def _generate_unique_uri():
    """Generates a unique uri for the chat session."""
    return str(uuid4()).replace('-', '')[:15]


class ChatSessionManager(models.Manager):
    def get_queryset(self):
        return super(ChatSessionManager, self).get_queryset().select_related()

    def user_session(self, user):
        queryset_first = self.get_queryset().filter(
            pk__in=ChatSessionMember.objects.filter(user=user).values_list('chat_session', flat=True).distinct())
        queryset = (queryset_first | self.get_queryset().filter(owner=user)).distinct().annotate(message_created_at=Max('messages__created_at'))
        return queryset

    def private(self, owner, user):
        """
        You can add private chat one by one.
        :param owner: User first time create the chat
        :param user: User add to this chat
        :return: ChatSession instance
        """
        username_first = owner.username
        username_second = user.username
        queryset = self.get_queryset().filter(Q(type=ChatSessionTypeEnum.PRIVATE), Q(owner__username=username_first,
                                                                                     members__user__username=username_second))
        queryset = (queryset | self.get_queryset().filter(Q(type=ChatSessionTypeEnum.PRIVATE),
                                                          Q(owner__username=username_second,
                                                            members__user__username=username_first))).distinct()
        if queryset.exists():
            return queryset.first()
        chat_session = ChatSession.objects.create(
            owner=owner,
            type=ChatSessionTypeEnum.PRIVATE,
        )
        ChatSessionMember.objects.create(
            chat_session=chat_session,
            user=user,
        )
        return chat_session


class ChatSession(TrackableDateModel):
    """
    A Chat Session.

    The uri's are generated by taking the first 15 characters from a UUID
    """

    owner = models.ForeignKey(User, on_delete=models.PROTECT, verbose_name=_('Owner'))
    uri = models.URLField(default=_generate_unique_uri, verbose_name=_('Uri'))
    type = enum.EnumField(ChatSessionTypeEnum, default=ChatSessionTypeEnum.GROUP,
                          verbose_name=_('Type'))
    cover_obj = models.ForeignKey(FileField, null=True, blank=True, on_delete=models.PROTECT, verbose_name=_('Cover'))
    title = models.CharField(max_length=255, default='', blank=True, verbose_name=_('Title'))

    objects = ChatSessionManager()

    def __str__(self):
        return self.uri

    def full_clean(self, exclude=None, validate_unique=True):
        if self.type != ChatSessionTypeEnum.PRIVATE and (self.title == '' or self.title is None):
            raise ValidationError(
                {'title': [_('This filed is required.'), ], }, )

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(ChatSession, self).save(*args, **kwargs)

    def get_api_uri(self):
        return reverse('aparnik-api:chats:details', args=[self.uri])

    def get_api_mark_as_read_uri(self):
        return reverse('aparnik-api:chats:mark-as-read', args=[self.uri])

    def get_member_api_uri(self):
        return reverse('aparnik-api:chats:members', args=[self.uri])

    def get_messages_api_uri(self):
        return reverse('aparnik-api:chats:messages', args=[self.uri])

    def get_messages_create_api_uri(self):
        return reverse('aparnik-api:chats:messages-create', args=[self.uri])


class ChatSessionMessage(TrackableDateModel):
    """Store messages for a session."""

    user = models.ForeignKey(User, on_delete=models.PROTECT)
    chat_session = models.ForeignKey(
        ChatSession, related_name='messages', on_delete=models.PROTECT
    )
    message = models.TextField(default='', max_length=2000)
    file_obj = models.ForeignKey(FileField, null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('File'))

    def get_api_uri(self):
        return reverse('aparnik-api:chats:messages-details', args=[self.pk])


class ChatSessionMemberManager(models.Manager):
    def get_queryset(self):
        return super(ChatSessionMemberManager, self).get_queryset().select_related()


class ChatSessionMember(TrackableDateModel):
    """Store all users in a chat session."""

    chat_session = models.ForeignKey(
        ChatSession, related_name='members', on_delete=models.PROTECT
    )
    user = models.ForeignKey(User, on_delete=models.PROTECT)

    objects = ChatSessionMemberManager()

    def get_api_uri(self):
        return reverse('aparnik-api:chats:members-details', args=[self.pk])


def post_save_chat_session_message_receiver(sender, instance, created, *args, **kwargs):
    if created:
        from .tasks import send_chat_message
        send_chat_message.delay(instance.pk)


post_save.connect(post_save_chat_session_message_receiver, sender=ChatSessionMessage)


class ChatMessageNotificationManager(models.Manager):
    def get_queryset(self):
        return super(ChatMessageNotificationManager, self).get_queryset().select_related()

    def active(self):
        return self.get_queryset()

    def mark_as_read_chat_session(self, user, chat_session):
        count = self.unread_chat_session(user, chat_session).update(is_read=True)
        return count

    def unread_chat_session(self, user, chat_session):
        return self.active().filter(chat_message__chat_session=chat_session, is_read=False,
                                                        user=user)

    def unread_count_chat_session(self, user, chat_session):
        return self.unread_chat_session(user, chat_session).count()

    def unread(self, user):
        return self.active().filter(user=user, is_read=False)

    def unread_count(self, user):
        return self.active().filter(user=user, is_read=False).count()


class ChatMessageNotification(TrackableDateModel):
    """Store all users notification in a chat message."""

    chat_message = models.ForeignKey(
        ChatSessionMessage, related_name='message_notifications', on_delete=models.PROTECT
    )
    user = models.ForeignKey(User, on_delete=models.PROTECT)
    is_read = models.BooleanField(default=False, verbose_name=_('Is read'))

    objects = ChatMessageNotificationManager()




# Generated by Django 2.2 on 2019-12-29 22:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('messaging', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='messaging',
            name='uri',
            field=models.CharField(max_length=255, null=True, verbose_name='URI'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db.models import Count, Q
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView

from aparnik.packages.shops.orders.models import Order
from aparnik.contrib.basemodels.api.views import BaseModelSortAPIView
from .serializers import ProductListSerializer
from ..models import Product


class ProductSortAPIView(BaseModelSortAPIView):
    permission_classes = [AllowAny]

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(ProductSortAPIView, self).__init__(*args, **kwargs)
        command_dict = {
            'buy_count': {
                'label': 'تعداد خرید',
                'queryset_filter': Q(),
                'annotate_command': {'count': Count('orderitem_set__order_obj', filter=Order.query_success('orderitem_set__order_obj'))},
                'key_sort': 'count',
            },
            'my_products': {
                'label': 'محصولات من',
                'queryset_filter': Q(),
                'annotate_command': {},
                'key_sort': 'my_product_count',
            }
        }

        self.command_dict.update(command_dict)


class ProductListAPIView(ListAPIView):
    serializer_class = ProductListSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        user = self.request.user
        # queryset = Product.objects.active(user=user)
        queryset = Product.objects.active(user=user)
        sort_api = ProductSortAPIView(request=self.request)
        queryset = sort_api.get_query_sort(queryset)
        return queryset



# -*- coding: utf-8 -*-


from rest_framework import serializers

from ..models import Messaging
from aparnik.contrib.users.api.serializers import UserSummaryListSerializer


# Messaging
class MessagingListSerializer(serializers.ModelSerializer):
    source = serializers.SerializerMethodField()
    recipient = serializers.SerializerMethodField()
    channels = serializers.SerializerMethodField()
    extra_data = serializers.SerializerMethodField()

    class Meta:
        model = Messaging
        fields = [
            'source',
            'source_display_name',
            'uri',
            'recipient',
            'category',
            'action',
            'obj',
            'short_description',
            'channels',
            'is_read',
            'extra_data',
        ]

        read_only_fields = [
            'source',
            'recipient',
            'extra_data',
            'channels',
        ]

    def get_source(self, obj):
        # if obj.source:
        #     return UserSummaryListSerializer(obj.source, many=False, read_only=True, context={'request': })
        return getattr(obj.source, 'username', None)

    def get_recipient(self, obj):
        return getattr(obj.recipient, 'username', None)

    def get_channels(self, obj):
        return obj.channels

    def get_extra_data(self, obj):
        return obj.extra_data


# -*- coding: utf-8 -*-
import abc

import six
from kavenegar import *
from aparnik.settings import aparnik_settings


@six.add_metaclass(abc.ABCMeta)
class SMSAPI():

    def __init__(self, **kwargs):
        pass

    @abc.abstractmethod
    def otp(self, receptor, otp, app_signature, first_name, last_name):
        pass


class SMSAPIKavenegar(SMSAPI):

    def otp(self, receptor, otp, app_signature, first_name, last_name):
        try:
            if not app_signature:
                app_signature = 'Code:%s' % otp

            api = KavenegarAPI(aparnik_settings.SMS_API_KEY)
            params = {
                'receptor': receptor,
                'template': aparnik_settings.SMS_OTA_NAME,
                'token': otp,
                'token2': app_signature,
                'type': 'sms',  # sms vs call
            }
            response = api.verify_lookup(params)
            # print(response)

        except APIException as e:
            print(e)

        except HTTPException as e:
            print(e)



from django.conf.urls import url, include

from .views import TicketListAPIView, TicketCreateAPIView#, CourseDetailAPIView, CourseUserListAPIView, CourseCreateAPIView

app_name = 'tickets'

urlpatterns = [
    url(r'^$', TicketListAPIView.as_view(), name='list'),
    url(r'^(?P<ticket_id>\d+)/conversations/', include('aparnik.contrib.tickets.api.urls-conversations', namespace='conversations')),
    url(r'^create/$', TicketCreateAPIView.as_view(), name='create'),
    url(r'^admin/', include('aparnik.contrib.tickets.api.admin.urls', namespace='admin')),


]


from django.conf.urls import url, include
from aparnik.contrib.segments.api.views import BaseSegmentListAPIView, BaseSegmentModelListAPIView

app_name = 'buttons'

urlpatterns = [
    url(r'^$', BaseSegmentListAPIView.as_view(), name='list'),
    url(r'^(?P<segment_id>\d+)/$', BaseSegmentModelListAPIView.as_view(), name='list-model-segment'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-03 15:33


import aparnik.utils.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0004_auto_20181103_2233'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('orders', '0002_auto_20181026_1301'),
    ]

    operations = [
        migrations.CreateModel(
            name='CoSale',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('price', aparnik.utils.fields.PriceField(decimal_places=0, default=0, max_digits=20, verbose_name='Price')),
                ('order_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cosale_order', to='orders.Order', verbose_name='Order')),
                ('user_bought_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cosale_user_bought', to=settings.AUTH_USER_MODEL, verbose_name='User Bought')),
                ('user_co_sale_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cosale_user_co_sale', to=settings.AUTH_USER_MODEL, verbose_name='User Co Sale')),
            ],
            options={
                'verbose_name': 'Co Sale',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Co Sales',
            },
            bases=('basemodels.basemodel',),
        ),
        migrations.CreateModel(
            name='CoSaleHistory',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('status', models.CharField(choices=[('C', 'Cleared'), ('NC', 'Not Cleared'), ('RSHBR', 'The request for settlement has been received')], default='NC', max_length=10, verbose_name='Status')),
                ('cosale_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cosales.CoSale', verbose_name='Co Sale')),
            ],
            options={
                'verbose_name': 'Co Sale History',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Co Sale Histories',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



from django.conf.urls import url, include


app_name='bankgateways'

urlpatterns = [
    # url(r'^payments/', include('aparnik.packages.shops.payments.urls', namespace='payments')),
    # url(r'^orders/', include('aparnik.packages.shops.orders.urls', namespace='orders')),
    url(r'^zarinpals/', include('aparnik.packages.bankgateways.zarinpals.urls', namespace='zarinpals')),
    url(r'^bmi/', include('aparnik.packages.bankgateways.bmi.urls', namespace='bmi')),
]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import Review, Like, BaseReview
from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelStackedInline
from aparnik.utils.utils import is_app_installed
# from courses.models import Course

# Register your models here.


# class CourseFilter(admin.SimpleListFilter):
#     title = _('Course')
#     parameter_name = 'course'
#
#     def lookups(self, request, model_admin):
#         courses = BaseModel.objects.instance_of(Course).filter(id__in=model_admin.model.objects.all().values_list('model_obj', flat=True))
#         return [(c.id, c.title) for c in courses]
#         # You can also use hardcoded model name like "Country" instead of
#         # "model_admin.model" if this is not direct foreign key filter
#         # return [('yes', 'yes')]
#
#     def queryset(self, request, queryset):
#         if self.value():
#             return queryset.filter(model_obj__id__exact=self.value())
#         else:
#             return queryset


class BaseReviewInline(admin.StackedInline):
    model = BaseReview
    exclude = ['update_needed', 'model_obj', 'user_obj']
    # fields = ['title']
    fk_name = 'parent_obj'
    dynamic_raw_id_fields = ['files']
    extra = 1

    def __init__(self, *args, **kwargs):
        Klass = BaseReviewInline
        Klass_parent = BaseModelStackedInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


    def get_queryset(self, request):
        return BaseReview.objects.filter(parent_obj__isnull=False)


class BaseReviewAdmin(BaseModelAdmin):
    fields = ['title', 'description', 'is_published', 'user_obj']
    list_display = ['title', 'user_obj', 'is_published', get_update_at, ]
    # list_filter = ['is_published', CourseFilter]
    list_filter = ['is_published']
    search_fields = get_user_search_fields('user_obj') + ['title', 'created_at', ]
    exclude = []
    dynamic_raw_id_fields = []
    # readonly_fields = []
    # actions = []
    raw_id_fields = ['user_obj',]
    inlines = [
        BaseReviewInline,
    ]

    def __init__(self, *args, **kwargs):
        Klass = BaseReviewAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = BaseReview

    def get_queryset(self, request):
        return BaseReview.objects.filter(parent_obj__isnull=True)

    def save_formset(self, request, form, formset, change):
        instances = formset.save(commit=False)
        for instance in instances:
            # Do something with `instance`
            if instance.id is None:
                instance.model_obj = form.instance.model_obj
                instance.user_obj = request.user
            instance.save()
        formset.save_m2m()


class ReviewInline(BaseReviewInline):
    model_name = Review

    def get_queryset(self, request):
        return Review.objects.filter(parent_obj__isnull=False)


class ReviewAdmin(BaseReviewAdmin):
    fields = ['rate', 'model_obj',]
    list_filter = []
    list_display = ['rate', ]
    search_fields = []
    exclude = []
    dynamic_raw_id_fields = []        # list_display = ['rate']

    # search_fields = ['title', 'rate', 'user_obj__username', 'created_at']
    # list_filter = ['is_published', 'rate', 'created_at']
    # readonly_fields = []
    # actions = []
    raw_id_fields = ['user_obj', 'model_obj',]
    inlines = [ReviewInline]

    def __init__(self, *args, **kwargs):
        Klass = ReviewAdmin
        Klass_parent = BaseReviewAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Review

    def get_queryset(self, request):
        return Review.objects.filter(parent_obj__isnull=True)


class LikeAdmin(BaseModelAdmin):
    list_display = ['user_obj', 'review_obj', 'is_like', get_update_at, ]
    list_filter = []
    fields = []
    exclude = []
    dynamic_raw_id_fields = []
    search_fields = get_user_search_fields('user_obj') + ['id', ]
    raw_id_fields = ['user_obj', ]

    def __init__(self, *args, **kwargs):
        Klass = LikeAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Like


if is_app_installed('aparnik.contrib.segments'):

    from aparnik.contrib.segments.admin import SegmentAdmin
    from .models import BaseReviewSegment


    class BaseReviewSegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        exclude = []
        raw_id_fields = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = BaseReviewSegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = BaseReviewSegment


    admin.site.register(BaseReviewSegment, BaseReviewSegmentAdmin)

admin.site.register(Review, ReviewAdmin)
admin.site.register(Like, LikeAdmin)



from django.conf.urls import url, include
from channels.routing import ProtocolTypeRouter, URLRouter

from channels.security.websocket import AllowedHostsOriginValidator, OriginValidator

from aparnik.middlewares.token_auth import TokenAuthMiddlewareStack
from aparnik.contrib.messaging.consumers import MessagingConsumer

application = ProtocolTypeRouter({
    # Empty for now (http->django views is added by default)
    'websocket': AllowedHostsOriginValidator(
        TokenAuthMiddlewareStack(
            URLRouter(
                [
                    url(r'^messaging/', MessagingConsumer),
                ]
            )
        )
    )
})



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basemodel',
            options={'ordering': ['-created_at'], 'verbose_name': '\\u0645\\u062f\\u0644 \\u067e\\u0627\\u06cc\\u0647', 'verbose_name_plural': '\\u0645\\u062f\\u0644 \\u0647\\u0627\\u06cc \\u067e\\u0627\\u06cc\\u0647'},
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='\\u0633\\u0627\\u062e\\u062a\\u0647 \\u0634\\u062f\\u0647 \\u062f\\u0631'),
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='review_average',
            field=models.FloatField(blank=True, null=True, verbose_name='\\u0645\\u06cc\\u0627\\u0646\\u06af\\u06cc\\u0646 \\u0646\\u0638\\u0631\\u0627\\u062a'),
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='update_at',
            field=models.DateTimeField(auto_now=True, verbose_name='\\u0628\\u0631\\u0648\\u0632\\u0631\\u0633\\u0627\\u0646\\u06cc \\u0634\\u062f\\u0647 \\u062f\\u0631'),
        ),
    ]



from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer
from aparnik.contrib.basemodels.api.serializers import BaseModelListSerializer, BaseModelDetailSerializer, ModelListPolymorphicSerializer
from ..models import Voucher


class VoucherListSerializer(BaseModelListSerializer):

    def __init__(self, *args, **kwargs):
        super(VoucherListSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Voucher
        fields = BaseModelListSerializer.Meta.fields + [
            'quantity',
            'expire_at',
            'is_active',
            'is_spent',
            'price',
            'price_string',
        ]


# ProductShare details
class VoucherDetailsSerializers(VoucherListSerializer, BaseModelDetailSerializer):

    def __init__(self, *args, **kwargs):
        super(VoucherDetailsSerializers, self).__init__(*args, **kwargs)

    class Meta:
        model = Voucher
        fields = VoucherListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [

        ]



# -*- coding: utf-8 -*-


from django.conf.urls import url
from .views import invoice


# Create your views here.
app_name = 'orders'

urlpatterns = [
    url(r'^(?P<uuid>[0-9a-f-]+)/invoice/$', invoice, name='invoice'),
]



"""Celery tasks."""

from __future__ import absolute_import, unicode_literals

import time

from celery import shared_task

from aparnik.contrib.chats.api.serializers import ChatSessionMessageDetailsSerializer
from aparnik.contrib.contactus.api.serializers import ContactUsListSerializer
from aparnik.contrib.users.models import User
from aparnik.contrib.contactus.models import *
from aparnik.contrib.messaging.utils import notify
from aparnik.utils.utils import get_request


@shared_task
def send_contact_us_message(contact_us_id):
    """Send chat message create notification and etc via a channel to celery."""
    time.sleep(0.2)
    contact_us = ContactUs.objects.get(pk=contact_us_id)
    serializer = ContactUsListSerializer(contact_us, many=False, read_only=True, context={'request': get_request()})
    users = User.objects.admins()
    for user in users:
        notify(
            uri=contact_us.phone,
            source=User.objects.first(),
            source_display_name=contact_us.fullname,
            recipient=user,
            action='Create',
            obj=contact_us,
            short_description='You a new contact request',
            extra_data={
                'uri': contact_us.phone,
                'message': serializer.data,
            },
            channels=['push'],
            silent=True,
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0003_auto_20181026_1131'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basecourse',
            options={'verbose_name': '\\u062f\\u0648\\u0631\\u0647 \\u067e\\u0627\\u06cc\\u0647', 'verbose_name_plural': '\\u062f\\u0648\\u0631\\u0647 \\u067e\\u0627\\u06cc\\u0647'},
        ),
        migrations.AlterModelOptions(
            name='course',
            options={'verbose_name': '\\u062f\\u0648\\u0631\\u0647', 'verbose_name_plural': '\\u062f\\u0648\\u0631\\u0647'},
        ),
        migrations.AlterModelOptions(
            name='coursefile',
            options={'verbose_name': '\\u0641\\u0627\\u06cc\\u0644', 'verbose_name_plural': '\\u0641\\u0627\\u06cc\\u0644 \\u0647\\u0627'},
        ),
        migrations.AlterModelOptions(
            name='coursesegment',
            options={'verbose_name': '\\u0628\\u062e\\u0634 \\u062f\\u0648\\u0631\\u0647', 'verbose_name_plural': '\\u0628\\u062e\\u0634 \\u062f\\u0648\\u0631\\u0647 \\u0647\\u0627'},
        ),
        migrations.AlterModelOptions(
            name='coursesummary',
            options={'verbose_name': '\\u062e\\u0644\\u0627\\u0635\\u0647 \\u062f\\u0648\\u0631\\u0647', 'verbose_name_plural': '\\u062e\\u0644\\u0627\\u0635\\u0647 \\u062f\\u0648\\u0631\\u0647'},
        ),
        migrations.AlterField(
            model_name='basecourse',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='\\u062a\\u0648\\u0636\\u06cc\\u062d\\u0627\\u062a'),
        ),
        migrations.AlterField(
            model_name='course',
            name='banner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='course_banner', to='filefields.FileField', verbose_name='\\u0628\\u0646\\u0631'),
        ),
        migrations.AlterField(
            model_name='course',
            name='category_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='categories.Category', verbose_name='\\u062f\\u0633\\u062a\\u0647 \\u0628\\u0646\\u062f\\u06cc'),
        ),
        migrations.AlterField(
            model_name='course',
            name='cover',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='course_cover', to='filefields.FileField', verbose_name='\\u062a\\u0635\\u0648\\u06cc\\u0631 \\u062c\\u0644\\u062f'),
        ),
        migrations.AlterField(
            model_name='course',
            name='dependency_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='dependency', to='courses.Course', verbose_name='\\u0648\\u0627\\u0628\\u0633\\u062a\\u06af\\u06cc'),
        ),
        migrations.AlterField(
            model_name='course',
            name='is_free',
            field=models.BooleanField(default=False, verbose_name='\\u0631\\u0627\\u06cc\\u06af\\u0627\\u0646\\u061f'),
        ),
        migrations.AlterField(
            model_name='course',
            name='parent_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='courses.Course', verbose_name='\\u0648\\u0627\\u0644\\u062f'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='\\u062a\\u0627\\u0631\\u06cc\\u062e \\u0627\\u0646\\u062a\\u0634\\u0627\\u0631'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_day',
            field=models.CharField(blank=True, choices=[('1', '\\u0634\\u0646\\u0628\\u0647'), ('2', '\\u06cc\\u06a9\\u0634\\u0646\\u0628\\u0647'), ('3', '\\u062f\\u0648\\u0634\\u0646\\u0628\\u0647'), ('4', '\\u0633\\u0647 \\u0634\\u0646\\u0628\\u0647'), ('5', '\\u0686\\u0647\\u0627\\u0631\\u0634\\u0646\\u0628\\u0647'), ('6', '\\u067e\\u0646\\u062c \\u0634\\u0646\\u0628\\u0647'), ('7', '\\u062c\\u0645\\u0639\\u0647')], max_length=1, null=True, verbose_name='\\u0631\\u0648\\u0632'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_month',
            field=models.CharField(blank=True, choices=[('1', '\\u0641\\u0631\\u0648\\u0631\\u062f\\u06cc\\u0646'), ('2', '\\u0627\\u0631\\u062f\\u06cc\\u0628\\u0647\\u0634\\u062a'), ('3', '\\u062e\\u0631\\u062f\\u0627\\u062f'), ('4', '\\u062a\\u06cc\\u0631'), ('5', '\\u0645\\u0631\\u062f\\u0627\\u062f'), ('6', '\\u0634\\u0647\\u0631\\u06cc\\u0648\\u0631'), ('7', '\\u0645\\u0647\\u0631'), ('8', '\\u0622\\u0628\\u0627\\u0646'), ('9', '\\u0622\\u0630\\u0631'), ('10', '\\u062f\\u06cc'), ('11', '\\u0628\\u0647\\u0645\\u0646'), ('12', '\\u0627\\u0633\\u0641\\u0646\\u062f')], max_length=2, null=True, verbose_name='\\u0645\\u0627\\u0647'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_week',
            field=models.CharField(blank=True, choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4')], max_length=1, null=True, verbose_name='\\u0647\\u0641\\u062a\\u0647'),
        ),
        migrations.AlterField(
            model_name='course',
            name='sort',
            field=models.IntegerField(default=0, verbose_name='\\u0645\\u0631\\u062a\\u0628 \\u0633\\u0627\\u0632\\u06cc'),
        ),
        migrations.AlterField(
            model_name='course',
            name='teacher_obj',
            field=models.ManyToManyField(blank=True, to='teachers.Teacher', verbose_name='\\u0645\\u062f\\u0631\\u0633\\u06cc\\u0646'),
        ),
        migrations.AlterField(
            model_name='coursefile',
            name='course_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='courses.Course', verbose_name='\\u062f\\u0648\\u0631\\u0647'),
        ),
        migrations.AlterField(
            model_name='coursefile',
            name='sort',
            field=models.IntegerField(default=0, verbose_name='\\u0645\\u0631\\u062a\\u0628 \\u0633\\u0627\\u0632\\u06cc'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='course',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='total_times', to='courses.BaseCourse', verbose_name='\\u062f\\u0648\\u0631\\u0647'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='file_count',
            field=models.PositiveIntegerField(default=0, verbose_name='\\u062a\\u0639\\u062f\\u0627\\u062f \\u0641\\u0627\\u06cc\\u0644 \\u0647\\u0627'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='file_count_preview',
            field=models.PositiveIntegerField(default=0, verbose_name='\\u062a\\u0639\\u062f\\u0627\\u062f \\u0641\\u0627\\u06cc\\u0644 \\u0647\\u0627\\u06cc \\u067e\\u06cc\\u0634 \\u0646\\u0645\\u0627\\u06cc\\u0634'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='type',
            field=models.CharField(choices=[('M', '\\u0641\\u06cc\\u0644\\u0645'), ('V', '\\u0635\\u062f\\u0627'), ('P', '\\u067e\\u06cc \\u062f\\u06cc \\u0627\\u0641'), ('I', '\\u0639\\u06a9\\u0633')], max_length=1, verbose_name='\\u0646\\u0648\\u0639'),
        ),
    ]



# Generated by Django 2.2 on 2020-02-14 18:03

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('chats', '0003_auto_20200210_1451'),
    ]

    operations = [
        migrations.CreateModel(
            name='ChatMessageNotification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('update_at', models.DateTimeField(auto_now=True)),
                ('is_read', models.BooleanField(default=False, verbose_name='Is read')),
                ('chat_message', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='message_notifications', to='chats.ChatSessionMessage')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.contrib.auth.models import AnonymousUser
from django.utils.translation import ugettext_lazy as _

from channels.generic.websocket import AsyncJsonWebsocketConsumer
from channels.db import database_sync_to_async
from channels.exceptions import DenyConnection

import json

class MessagingConsumer(AsyncJsonWebsocketConsumer):
    user = AnonymousUser()

    def __init__(self, *args, **kwargs):
        super(MessagingConsumer, self).__init__( *args, **kwargs)

    async def connect(self):
        self.user = self.scope['user']
        if self.user == AnonymousUser():
            raise DenyConnection(_("Invalid User"))

        await self.accept()

        # add to room
        rooms = self.add_to_rooms()
        rooms = await rooms
        rooms.add('broadcast')
        rooms.add(self.user.username)
        for room_id in list(rooms):
            await self.channel_layer.group_add(
                room_id,
                self.channel_name,
            )

        # Store which rooms the user has send on this connection

    async def receive_json(self, content, **kwargs):
        # await self.send_json({"ty": "test"})
        # await self.send_json(content)
        pass

    async def disconnect(self, code):
        """
        Called when the WebSocket closes for any reason.
        """
        # Leave all the rooms we are still in
        # for room_id in list(self.rooms):
        #     self.rooms.discard(room_id)
        #     await self.channel_layer.group_discard(
        #         room_id,
        #         self.channel_name,
        #     )
        pass

    ##### Command helper methods called by receive_json

    # These helper methods are named by the types we send - so chat.join becomes chat_join
    """
    format event:
    :type
    :data
    """
    async def send_data(self, event):
        """
        Called when someone has send data.
        """
        # Send a message down to the client
        params = event['data']
        await self.send_json(
            params,
        )

    # add to room
    @database_sync_to_async
    def add_to_rooms(self):
        l = set()
        from aparnik.contrib.chats.models import ChatSession
        for room_id in ChatSession.objects.user_session(user=self.user).values_list('uri', flat=True):
            l.add(room_id)
        return l


# class MessagingConsumer(AsyncConsumer):
#     def __init__(self, *args, **kwargs):
#         super(MessagingConsumer, self).__init__(*args, **kwargs)
#         self.user = None
#
#     async def websocket_connect(self, event):
#         self.user = self.scope['user']
#         if self.scope['user'] == AnonymousUser():
#             raise DenyConnection(_("Invalid User"))
#
#         await self.send({
#             'type': 'websocket.accept',
#         })
#         # other = None
#         # # other = self.scope['url_route']['kwargs']['username']
#         # print(me, other)
#
#     async def websocket_receive(self, event):
#         print('receive', event)
#         data = await self.deserialize(event)
#         # bar asas data pasokh monaseb
#
#     async def websocket_disconnect(self, event):
#         print('disconnect', event)
#
#
#     async def deserialize(self, data):
#         client_text = data.get('text', None)
#         if client_text is not None:
#             loaded_dict_data = json.loads(client_text)
#             return loaded_dict_data
#         return {}
#
#     @database_sync_to_async
#     def get_thread(self, user, other_username):
#         # return get_or_new(user, other_username)[0]
#         return 1


# -*- coding: utf-8 -*-


from django.contrib import admin
from django.db.models import Q
from django.contrib.admin import SimpleListFilter

from jalali_date import datetime2jalali
from rangefilter.filter import DateRangeFilter, DateTimeRangeFilter
from import_export.admin import ExportActionMixin

from aparnik.settings import aparnik_settings
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.admin import get_user_search_fields

from .models import Counter

from import_export import resources, fields
# Register your models here.

class CounterResource(resources.ModelResource):
    model = fields.Field()

    class Meta:
        model = Counter
        fields = [
            'id',
            'user_obj__username',
            'model',
            'model_obj__id',
            'update_at',
        ]

    def dehydrate_model(self, counter):
        return counter.model_obj.get_real_instance().get_title()



class ModelFilter(SimpleListFilter):
    title = 'model' # or use _('country') for translated title
    parameter_name = 'model'

    def lookups(self, request, model_admin):
        models = set(['CourseFile', 'Course'])
        return [(c, c) for c in models]

    def queryset(self, request, queryset):
        from aparnik.packages.educations.courses.models import Course, CourseFile
        from aparnik.contrib.basemodels.models import BaseModel
        if self.value() == 'CourseFile':
            return queryset.filter(model_obj_id__in=BaseModel.objects.filter(Q(instance_of=CourseFile)))
        if self.value() == 'Course':
            return queryset.filter(model_obj_id__in=BaseModel.objects.filter(Q(instance_of=Course)))
        return queryset

def get_title(obj):
    return obj.model_obj.get_real_instance().get_title()

get_title.short_description = 'عنوان'

# def get_update_at(obj):
#     return datetime2jalali(obj.update_at).strftime(aparnik_settings.JALALI_FORMAT)

#
# get_update_at.admin_order_field = 'update_at'
# get_update_at.short_description = 'بروز شده'


class CounterAdmin(ExportActionMixin, BaseModelAdmin):
    resource_class = CounterResource
    fields = ['user_obj', 'model_obj', 'action']
    list_display = ['user_obj', 'model_obj', get_title, 'action', 'update_at']
    list_filter = ['action', ('update_at', DateTimeRangeFilter), ModelFilter,]
    search_fields = get_user_search_fields('user_obj') + ['model_obj__id',]
    exclude = []
    raw_id_fields = ['user_obj', 'model_obj']
    dynamic_raw_id_fields = []
    readonly_fields = ['update_at']

    def __init__(self, *args, **kwargs):
        Klass = CounterAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Counter


admin.site.register(Counter, CounterAdmin)



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Sum, Count
from django.db.models.signals import post_save
from django.db.models import Q
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils.timezone import now

from decimal import Decimal
import uuid

from aparnik.utils.utils import round, field_with_prefix
from aparnik.utils.fields import PriceField
from aparnik.utils.formattings import formatprice
from aparnik.settings import Setting
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.addresses.models import UserAddress
from aparnik.contrib.notifications.models import NotificationForSingleUser, Notification
from aparnik.packages.shops.products.models import Product
from aparnik.packages.shops.coupons.models import Coupon

User = get_user_model()


# Create your models here.


class OrderManager(BaseModelManager):

    def get_queryset(self):
        return super(OrderManager, self).get_queryset()

    def get_this_user(self, user, custom_type=None):
        if not user.is_authenticated:
            return Order.objects.none()
        queryset = self.get_queryset().filter(user=user)
        if custom_type:
            queryset = queryset.filter(
                items__product_obj__polymorphic_ctype=ContentType.objects.get_for_model(custom_type))
        return queryset

    def get_this_user_bought(self, user, custom_type=None):
        return self.get_this_user(user, custom_type).filter(Order.query_success())

    def get_order_product(self, user, product):
        from aparnik.packages.shops.subscriptions.models import Subscription
        buy = self.get_this_user(user=user).filter(
            Order.query_success()
        ).filter(
            Q(items__product_obj=product) |
            (
                    Q(
                        subscriptionorder__expire_at__gte=now(),
                        subscriptionorder__subscription_obj__type=Subscription.TYPE_ALL
                    ) |
                    Q(
                        subscriptionorder__expire_at__gte=now(),
                        subscriptionorder__subscription_obj__type=Subscription.TYPE_CUSTOM,
                        subscriptionorder__subscription_obj__products=product
                    )
            )
        )
        if buy:
            return buy.first()
        return None

    def get_order_products(self, products):
        orders = self.get_queryset().filter(
            Order.query_success()).filter(items__product_obj__id__in=products)
        return orders


class Order(BaseModel):
    # Status:
    STATUS_CANCEL = 'c'
    STATUS_COMPLETE = 'co'
    STATUS_DISPUTED = 'd'
    STATUS_CHALLENGED = 'ch'
    STATUS_WAITING = 'w'
    STATUS_PAID = 'pa'
    STATUS_PAID_BY_WEBSITE = 'paw'
    STATUS_CHOICES = (
        (STATUS_WAITING, _('Waiting')),
        (STATUS_PAID, _('Paid')),
        (STATUS_PAID_BY_WEBSITE, _('Paid by website')),
        (STATUS_CANCEL, _('Cancel')),
        (STATUS_COMPLETE, _('Complete')),
        (STATUS_DISPUTED, _('Disputed')),
        (STATUS_CHALLENGED, _('Challenged')),
    )

    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    coupon = models.ForeignKey(Coupon, related_name='orders', on_delete=models.CASCADE, null=True, blank=True,
                               verbose_name=_('Coupon'))
    status = models.CharField(max_length=5, default=STATUS_WAITING, choices=STATUS_CHOICES, verbose_name=_('Status'))
    address_obj = models.ForeignKey(UserAddress, on_delete=models.CASCADE, null=True, blank=True,
                                    verbose_name=_('Order Address'))
    uuid = models.UUIDField(default=uuid.uuid4, unique=True, editable=False, verbose_name=_('Reference ID'))
    postal_cost_value = PriceField(big=True, default=0, verbose_name=_('Postal Cost'))
    is_sync_with_websites = models.BooleanField(default=False, verbose_name=_('Is sync with websites'))

    objects = OrderManager()

    class Meta:
        ordering = ('-created_at',)
        verbose_name = _('Order')
        verbose_name_plural = _('Orders')

    def __str__(self):
        return '{}'.format(self.id)

    @staticmethod
    def query_success(prefix='', user=None):
        from aparnik.packages.shops.subscriptions.models import Subscription
        field_subscription = field_with_prefix('subscriptionorder', prefix)
        field_status = field_with_prefix('status', prefix)
        field_user = field_with_prefix('user', prefix)

        query_filter_status = Q(Q(**{field_status: Order.STATUS_COMPLETE}) | \
                                Q(**{field_status: Order.STATUS_PAID}) | \
                                Q(**{field_status: Order.STATUS_PAID_BY_WEBSITE}))
        # TODO: چطوری میتونیم یک کوئری مثل منیجر در حالت کلی بنویسیم که بتونیم متوجه بشیم که این محصول خریداری شده یا نه
        # query_filter_subscription = Q(
        #     Q(
        #         **{field_subscription + '__expire_at__gte': now(),
        #            field_subscription + '__subscription_obj__type': Subscription.TYPE_ALL,
        #            }
        #     ) |
        #     Q(
        #         **{
        #             field_subscription + '__expire_at__gte': now(),
        #             field_subscription + '__subscription_obj__type': Subscription.TYPE_CUSTOM,
        #             # subscriptionorder__subscription_obj__products=product,
        #         }
        #     )
        # )

        query_filter_user = Q()
        if user:
            query_filter_user = Q(Q(**{field_user: user}))

        return Q(query_filter_status, query_filter_user)

    @staticmethod
    def query_waiting(prefix=''):
        field = field_with_prefix('status', prefix)
        return Q(**{field: Order.STATUS_WAITING})

    @staticmethod
    def query_cancel(prefix=''):

        field = field_with_prefix('status', prefix)
        return Q(**{field: Order.STATUS_CANCEL})

    @staticmethod
    def query_date(prefix='', start_date=None, end_date=None):

        if not start_date:
            start_date = now().date()
        if not end_date:
            end_date = now()
        field = field_with_prefix('payments__update_at__range', prefix)
        return Q(**{field: [start_date, end_date]})

    @staticmethod
    def sort_buy_wallet(return_key='buy_wallet', prefix=''):
        from aparnik.packages.shops.payments.models import Payment
        sort = {
            return_key: {
                'label': 'از طریق کیف پول',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count(
                            field_with_prefix('payments', prefix),
                            filter=Q(
                                Order.query_success(prefix),
                                Q(**{field_with_prefix('payments__status', prefix): Payment.STATUS_COMPLETE}),
                                Q(**{field_with_prefix('payments__method', prefix): Payment.METHOD_WALLET}),
                            )
                        )
                },
                'key_sort': 'sort_count',
            }
        }
        return sort

    @staticmethod
    def sort_buy_bank(return_key='buy_bank', prefix=''):
        from aparnik.packages.shops.payments.models import Payment
        from aparnik.packages.bankgateways.zarinpals.models import Bank
        sort = {
            return_key: {
                'label': 'از طریق بانک',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count(
                            field_with_prefix('payments__bank', prefix),
                            filter=Q(
                                Order.query_success(prefix),
                                Q(**{field_with_prefix('payments__status', prefix): Payment.STATUS_COMPLETE}),
                                Q(**{field_with_prefix('payments__method', prefix): Payment.METHOD_BANK}),
                                Q(**{field_with_prefix('payments__bank__status', prefix): Bank.TRANSACTION_SUCCESS}),
                            )
                        )
                },
                'key_sort': 'sort_count',
            }
        }
        return sort

    @staticmethod
    def sort_buy(return_key='buy_count', prefix=''):

        sort = {
            return_key: {
                'label': 'تعداد خرید',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count(
                            'id',
                            filter=Order.query_success(prefix)
                        )
                },

                'key_sort': 'sort_count',
            }
        }

        return sort

    @staticmethod
    def sort_buy_waiting(return_key='buy_waiting_count', prefix=''):
        sort = {
            return_key: {
                'label': 'تعداد خرید در انتظار پرداخت',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count(
                            'id',
                            filter=Order.query_waiting(prefix)
                        )
                },

                'key_sort': 'sort_count',
            }
        }
        return sort
        # def get_api_uri(self):

    #     return reverse('aparnik-api:shops:orders:detail', args=[self.id])

    def get_api_request_pay_uri(self):
        return reverse('aparnik-api:shops:orders:request-pay', args=[self.id])

    def get_api_coupon_uri(self):
        return reverse('aparnik-api:shops:orders:coupon', args=[self.id])

    def get_api_add_address_uri(self):
        return reverse('aparnik-api:shops:orders:add-address', args=[self.id])

    def get_api_add_item_uri(self):
        return reverse('aparnik-api:shops:orders:add-item', args=[self.id])

    def get_pay_uri(self):
        return reverse('aparnik:shops:orders:invoice', args=[self.uuid])

    # Mablaghe faktor
    def get_total_cost_order(self):
        return round(sum(item.get_cost() for item in self.items.all()))

    @property
    def total_cost_order_string(self):
        return '%s' % formatprice.format_price(self.get_total_cost_order())

    # mablaghe ghabele pardakht
    def get_total_cost(self):
        total_cost = self.get_total_cost_order()

        return round(total_cost + self.postal_cost - self.get_discount() - self.get_bon_price())

    @property
    def total_cost_string(self):
        return '%s' % formatprice.format_price(self.get_total_cost())

    # مبلغ به حروف
    @property
    def total_cost_to_word(self):
        return formatprice.format_price(self.get_total_cost(), str('%ic=t:%se=,:%cu=t:%StrTr=True'))

    @property
    def order_code(self):
        orderCode = ('%s' % (Setting.objects.get(key='ORDER_CODE_PREFIX')).get_value() + '%s' % str(self.id))
        return orderCode

    # takhfif
    def get_discount(self):
        total_cost = self.get_total_cost_order()
        discount = 0
        if self.coupon is not None:
            discount = self.coupon.calculate_discount(total_cost=total_cost)

        return round(discount)

    @property
    def postal_cost(self):
        if self.is_success:
            return self.postal_cost_value
        postal_cost = Setting.objects.get(key='POSTAL_COST').get_value()
        return postal_cost

    @property
    def postal_cost_string(self):
        postal_cost = self.postal_cost
        return '%s' % formatprice.format_price(postal_cost)

    @property
    def discount_string(self):
        return '%s' % formatprice.format_price(self.get_discount())

    def get_bon_price(self):
        from aparnik.packages.shops.vouchers.models import Voucher
        return Voucher.objects.price_accessible_on_order(order=self)

    @property
    def bon_price_string(self):
        return '%s' % formatprice.format_price(self.get_bon_price())

    @property
    def bon_quantity(self):
        from aparnik.packages.shops.vouchers.models import Voucher
        return Voucher.objects.quantities_accessible_on_order(order=self)

    @property
    def total_dollar(self):
        dollar = 0
        for item in self.items.all():
            dollar = dollar + (item.dollar * item.quantity)

        return dollar

    @property
    def total_dollar_string(self):
        return '$%s' % self.total_dollar

    @property
    def total_count_item(self):
        x = 0
        for item in self.items.all():
            x = x + item.quantity
        return x

    @property
    def has_permit_use_wallet(self):
        for item in self.items.all():
            if not item.product_obj.has_permit_use_wallet:
                return False

        return True

    def add_item(self, product, quantity=1, description=''):
        item = None
        try:
            item = OrderItem.objects.get(
                order_obj=self,
                product_obj=product,
            )
            item.price = product.price
            item.quantity = quantity
            item.description = description
        except:
            item = OrderItem.objects.create(
                order_obj=self,
                product_obj=product,
                price=product.price,
                quantity=quantity,
                description=description
            )
        item.save()
        return item

    # TODO: check the coupon or all of the thing check need before pay
    def is_allow_to_pay(self):
        if self.coupon is not None:
            try:
                status = Coupon.objects.status(code=self.coupon.code, user=self.user, order=self)
                if status == Coupon.COUPON_STATUS_OK:
                    return True
                return False
            except:
                return False
        else:
            return True

    def pay_success(self, status=None):
        wallet_id = Product.objects.get_wallet().id
        if not status:
            status = Order.STATUS_PAID

        self.status = status

        if self.coupon is not None:
            self.coupon.redeem(user=self.user)

        for item in self.items.all():
            if item.product_obj.id == wallet_id:
                self.user.wallet = self.user.wallet + (item.product_obj.price * item.quantity)
                self.user.save()
            item.product_obj.get_real_instance().buy_success(order=self)

        self.postal_cost_value = Setting.objects.get(key='POSTAL_COST').get_value()

        self.save()

        if self.get_total_cost_order() > 0:
            NotificationForSingleUser.objects.send_notification(
                users=[self.user], type=Notification.NOTIFICATION_SUCCESS,
                title='خرید شما با موفقیت انجام شد.',
                description='فاکتور خرید شما به مبلغ %s با موفقیت پرداخت گردید.' % self.total_cost_string,
                model_obj=self,
            )

    def cancel(self):
        self.status = Order.STATUS_CANCEL
        self.save()

    @property
    def is_success(self):
        if self.status == Order.STATUS_COMPLETE or self.status == Order.STATUS_PAID or self.status == Order.STATUS_PAID_BY_WEBSITE:
            return True
        return False

    @property
    def earn_bon_if_buy(self):
        earn_bon = 0
        for item in self.items.all():
            earn_bon += item.product_obj.aparnik_bon_return * item.quantity
        return earn_bon

    @property
    def earn_bon_if_buy_value(self):
        price = round(self.earn_bon_if_buy * Setting.objects.get(key='APARNIK_BON_VALUE').get_value())
        return price

    @property
    def earn_bon_if_buy_value_string(self):
        price = self.earn_bon_if_buy_value
        return '%s' % formatprice.format_price(price)


def post_save_order_receiver(sender, instance, created, *args, **kwargs):

    from .tasks import send_order_message
    send_order_message.delay(instance.pk, created)

    from aparnik.packages.shops.vouchers.models import Voucher
    Voucher.objects.change_order(instance)

    from aparnik.packages.shops.cosales.models import CoSale
    CoSale.objects.change_order(order=instance)


post_save.connect(post_save_order_receiver, sender=Order)


class OrderItemManager(BaseModelManager):

    def get_queryset(self):
        return super(OrderItemManager, self).get_queryset()

    def get_this_user(self, user):
        if not user.is_authenticated:
            return OrderItem.objects.none()
        return self.get_queryset().filter(order_obj__user=user)


class OrderItem(BaseModel):
    order_obj = models.ForeignKey(Order, related_name='items', on_delete=models.CASCADE, verbose_name=_('Order'))
    product_obj = models.ForeignKey(Product, related_name='orderitem_set', on_delete=models.CASCADE,
                                    verbose_name=_('Product'))
    price = PriceField(big=True, verbose_name=_('Price'))
    is_tax = models.BooleanField(default=False, verbose_name=_('Is Tax'))
    quantity = models.PositiveIntegerField(default=1, verbose_name=_('Quantity'))
    description = models.TextField(default='', blank=True, null=True, verbose_name=_('Description'))

    objects = OrderItemManager()

    def __str__(self):
        return '{}'.format(self.id)

    def get_cost(self):

        return self.price_total_without_tax + self.price_tax_total

    def get_api_remove_item_uri(self):
        return reverse('aparnik-api:shops:orders:remove-item', args=[self.order_obj.id, self.id])

    @property
    def price_string(self):
        return '%s' % formatprice.format_price(self.price)

    @property
    def price_total_without_tax(self):
        if self.order_obj.status == Order.STATUS_WAITING and self.price != self.product_obj.price:
            self.price = self.product_obj.price
            self.save()

        if self.order_obj.status == Order.STATUS_WAITING and self.is_tax != self.product_obj.is_tax:
            self.is_tax = self.product_obj.is_tax
            self.save()
        price = self.price * self.quantity
        return price

    @property
    def price_total_without_tax_string(self):
        return '%s' % formatprice.format_price(self.price_total_without_tax)

    @property
    def price_complications(self):
        if self.is_tax:
            return round(self.price_total_without_tax * (3 / Decimal('100')))
        return 0

    @property
    def price_complications_string(self):
        return '%s' % formatprice.format_price(self.price_complications)

    @property
    def price_tax(self):
        if self.is_tax:
            return round(self.price_total_without_tax * (6 / Decimal('100')))
        return 0

    @property
    def price_tax_string(self):
        return '%s' % formatprice.format_price(self.price_tax)

    @property
    def price_tax_total(self):
        if self.is_tax:
            return self.price_tax + self.price_complications
        return 0

    @property
    def price_tax_total_string(self):
        return '%s' % formatprice.format_price(self.price_tax_total)

    @property
    def price_total_string(self):
        return '%s' % formatprice.format_price(self.get_cost())

    @property
    def dollar(self):
        return self.product_obj.price_fabric if self.product_obj.currency == Product.CURRENCY_DOLLAR else 0

    @property
    def dollar_string(self):
        return '$%s' % self.dollar

    @property
    def aparnik_bon_return(self):
        return int(self.product_obj.aparnik_bon_return * self.quantity)

    @property
    def maximum_use_aparnik_bon(self):
        return int(self.product_obj.maximum_use_aparnik_bon * self.quantity)

    class Meta:
        ordering = ('created_at',)
        verbose_name = _('OrderItem')
        verbose_name_plural = _('OrderItems')



"""views.py."""

from json import loads

from django.http import JsonResponse
from django.contrib.auth import get_user_model
from django.views.generic import ListView, View
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.core.exceptions import ObjectDoesNotExist

from . import default_settings as settings

from .signals import notify
from .models import Messaging


class NotificationsView(ListView):
    """View for notifications for clients/mechanics."""

    model = Messaging
    context_object_name = 'notifications_list'

    paginate_by = settings.NOTIFICATIONS_PAGINATE_BY

    def get_queryset(self):
        """Filter notifications by currently logged in user."""
        queryset = super().get_queryset()

        return queryset.filter(recipient=self.request.user)


class GenerateNotification(View):
    """View to generate test notifications."""

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        """disable CSRF Verification."""
        return super(GenerateNotification, self).dispatch(request, *args, **kwargs)

    def add_access_control_headers(self, response):
        """Add headers for CORS."""
        response["Access-Control-Allow-Origin"] = "*"
        response["Access-Control-Allow-Methods"] = "POST, OPTIONS"
        response["Access-Control-Max-Age"] = "1000"
        response["Access-Control-Allow-Headers"] = "X-Requested-With, Content-Type"

    def post(self, request, *args, **kwargs):
        """Generate the notification."""
        data = loads(request.body)
        message = data['message']

        # Create/retrieve the user
        User = get_user_model()

        try:
            user = User.objects.get(username='demouser')
        except ObjectDoesNotExist:
            user = User.objects.create_user(
                username='demouser', email='example@gmail.com',
                password='mypassword'
            )

        # notification
        args = {
            'source': user, 'source_display_name': user.get_full_name(),
            'recipient': user, 'category': 'Quote', 'action': 'Sent',
            'obj': user.id,
            'short_description': 'You a message: {}'.format(message),
            'url': 'http://example.com', 'channels': ('websocket',)
        }
        notify.send(sender=self.__class__, **args)

        response = JsonResponse({'message': 'Notification generated'})

        self.add_access_control_headers(response)

        return response



# Generated by Django 2.2 on 2019-06-26 15:49

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('reviews', '0008_auto_20190613_1830'),
    ]

    operations = [
        migrations.AddField(
            model_name='review',
            name='user_obj_2',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


import aparnik.utils.utils
from django.db import migrations, models
import s3direct.fields


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0002_auto_20181025_1617'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='filefield',
            options={'verbose_name': '\\u0641\\u0627\\u06cc\\u0644 \\u0641\\u06cc\\u0644\\u062f', 'verbose_name_plural': '\\u0641\\u0627\\u06cc\\u0644 \\u0641\\u06cc\\u0644\\u062f'},
        ),
        migrations.AlterField(
            model_name='filefield',
            name='file_direct',
            field=models.FileField(blank=True, null=True, upload_to=aparnik.utils.utils.document_directory_path, verbose_name='\\u0641\\u0627\\u06cc\\u0644'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='file_s3',
            field=s3direct.fields.S3DirectField(blank=True, null=True, verbose_name='\\u0641\\u0627\\u06cc\\u0644'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='size',
            field=models.IntegerField(default=0, verbose_name='\\u0633\\u0627\\u06cc\\u0632 \\u0641\\u0627\\u06cc\\u0644'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='type',
            field=models.CharField(choices=[('M', '\\u0641\\u06cc\\u0644\\u0645'), ('V', '\\u0635\\u062f\\u0627'), ('P', '\\u067e\\u06cc \\u062f\\u06cc \\u0627\\u0641'), ('I', '\\u0639\\u06a9\\u0633')], max_length=1, verbose_name='\\u0646\\u0648\\u0639 \\u0641\\u0627\\u06cc\\u0644'),
        ),
    ]



from django.db import models
from django.core.validators import RegexValidator
from django.utils.translation import ugettext_lazy as _


class PhoneField(models.CharField):

    def __init__(self, max_length=255, validators=[], mobile=False, *args, **kwargs):
        if mobile:
            # validators = [RegexValidator(regex='^09([0-9]{9})$', message=_('mobile is not valid'), code='nomatch')]
            validators = [RegexValidator(regex='^0{2}[1-9]{1}[0-9]{0,2}[1-9][0-9]{2}[0-9]{3}[0-9]+$')]
        else:
            validators = [RegexValidator(regex='^0(?!0)\d{2}([0-9]{8})$', message=_('phone is not valid, please insert with code'), code='nomatch')]

        super(PhoneField, self).__init__(max_length=max_length, validators=validators, *args, **kwargs)

    # def deconstruct(self):
    #     name, path, args, kwargs = super(CommaSepField, self).deconstruct()
    #     # Only include kwarg if it's not the default
    #     if self.separator != ",":
    #         kwargs['separator'] = self.separator
    #     return name, path, args, kwargs


class PriceField(models.DecimalField):

    def __init__(self, big=False, max_digits=8, decimal_places=0, *args, **kwargs):
        if big:
            max_digits = 20
        super(PriceField, self).__init__(max_digits=max_digits, decimal_places=decimal_places, *args, **kwargs)


class NationalCodeField(models.CharField):

    def __init__(self, validators=[], max_length=10, birth_certificate=False, *args, **kwargs):

        validators = []

        if birth_certificate:

            validators = [RegexValidator(regex='^\d{1,10}$', message=_('Birth Certificate code is not valide'), code='nomatch')]
        else:

            validators = [RegexValidator(regex='^\d{10}$', message=_('National Code is not valide'), code='nomatch')]
        super(NationalCodeField, self).__init__(max_length=max_length, validators=validators, *args, **kwargs)


class PostalCodeField(models.CharField):

    def __init__(self, max_length=10, validators=[], *args, **kwargs):
        validators = [RegexValidator(regex='^\d{10}$', message=_('zip code is not valid'), code='nomatch')]
        super(PostalCodeField, self).__init__(max_length=max_length, validators=validators, *args, **kwargs)


# -*- coding: utf-8 -*-


from rest_framework.response import Response
from rest_framework.status import HTTP_201_CREATED, HTTP_400_BAD_REQUEST
from rest_framework.generics import (
    CreateAPIView,
    # DestroyAPIView,
    ListAPIView,
    # UpdateAPIView,
    RetrieveAPIView,
    # RetrieveUpdateAPIView
    )
from rest_framework.permissions import (
    AllowAny,
    IsAuthenticated,
    IsAdminUser,
    IsAuthenticatedOrReadOnly,

    )
from rest_framework.pagination import PageNumberPagination
from django.db.models import Q

from aparnik.contrib.invitation.models import Invite
from aparnik.contrib.invitation.api.serializers import InviteListSerializer, InviteDetailSerializer

class LargeResultsSetPagination(PageNumberPagination):
    page_size = 100
    page_size_query_param = 'page_size'
    max_page_size = 10


class InviteListAPIView(ListAPIView):
    serializer_class = InviteListSerializer
    permission_classes = [IsAuthenticated]
    pagination_class = LargeResultsSetPagination

    def get_queryset(self):
        user = self.request.user

        return Invite.objects.filter(invited_by=user)


class InviteDetailAPIView(RetrieveAPIView):
    queryset = Invite.objects.all()
    serializer_class = InviteDetailSerializer
    permission_classes = [IsAuthenticated]
    lookup_field = 'id'

    def get_queryset(self):
        user = self.request.user

        return Invite.objects.filter(invited_by=user)



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Max
from django.db.models.signals import post_save
from django.urls import reverse
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from django.utils.timezone import now
from django.contrib.auth import get_user_model
import datetime

from ckeditor_uploader.fields import RichTextUploadingField

from aparnik.contrib.settings.models import Setting
from aparnik.contrib.filefields.models import FileField
from aparnik.contrib.segments.models import BaseSegment, BaseSegmentManager
from aparnik.contrib.categories.models import Category
from aparnik.packages.educations.teachers.models import Teacher
from aparnik.packages.shops.products.models import Product, ProductManager
from aparnik.packages.shops.files.models import File, FileManager

User = get_user_model()


# Total Time Manager
class TotalTimeManager(models.Manager):

    def get_queryset(self):
        return super(TotalTimeManager, self).get_queryset()


# Total Time Model
class CourseSummary(models.Model):
    total_time_seconds = models.BigIntegerField(default=0, verbose_name=_('Time'))
    file_count = models.PositiveIntegerField(default=0, verbose_name=_('File Count'))
    file_count_preview = models.PositiveIntegerField(default=0, verbose_name=_('File Count Preview'))
    type = models.CharField(max_length=1, choices=FileField.FILE_TYPE, verbose_name=_('Type'))
    course = models.ForeignKey('BaseCourse', related_name='total_times', on_delete=models.CASCADE, verbose_name=_('Course'))

    objects = TotalTimeManager()

    class Meta:
        ordering = ('id',)
        verbose_name = _('Course Summary')
        verbose_name_plural = _('Courses Summary')

    def __str__(self):
        return '%s' % self.id

    @property
    def total_time(self):
        time = self.total_time_seconds
        hours = time / (60 * 60)
        minutes = (time % (60 * 60)) / 60
        seconds = (time % (60 * 60)) % 60
        v_t = "{}:{}:{}".format(hours, minutes, seconds)
        return v_t

    @property
    def type_icon_url(self):
        return FileField.type_icon_url_with_custom_type(self.type)


# Base Course Manager
class BaseCourseManager(ProductManager):

    def get_queryset(self):
        return super(BaseCourseManager, self).get_queryset()

    def active(self, user=None):
        return super(BaseCourseManager, self).active(user=user)


# Base Course Model
class BaseCourse(Product):
    description = RichTextUploadingField(null=True, blank=True, config_name='custom', verbose_name=_('Description'))

    objects = BaseCourseManager()

    class Meta:
        verbose_name = _('Base Course')
        verbose_name_plural = _('Bases Courses')

    def __str__(self):
        return '%s' % self.id

    def get_description(self):
        return self.description


# Course Manager
class CourseManager(BaseCourseManager):

    def get_queryset(self):
        return super(CourseManager, self).get_queryset().order_by('sort')

    def active(self, user=None):
        return super(CourseManager, self).active(user=user).filter(publish_date__lte=now())

    def get_this_user(self, user):
        # TODO: return course for this user
        return self.active()

    def courses(self):
        ids = []
        try:
            setting = Setting.objects.get(key='COURSE_IGNORE_IN_LIST_IDS')
            ids = [int(x.strip()) for x in setting.get_value().split(',')]
        except:
            pass

        return self.active().exclude(id__in=ids).filter(parent_obj=None, is_show_only_for_super_user=False)

    def childs(self, course_obj):
        return self.active().filter(parent_obj=course_obj)

    def childs_count(self, course_obj):
        return self.childs(course_obj).count()

    def children_deep(self, course=None, course_ids=None):
        if course_ids:
            courses_id = course_ids
        elif course:
            courses_id = [course.id]
        else:
            return Course.objects.none()

        course_children_ids = courses_id
        # get course children
        while True:
            course_children = Course.objects.active().filter(parent_obj__id__in=course_children_ids)

            if course_children.count() > 0:
                course_children_ids = list(course_children.values_list('id', flat=True))
                courses_id = courses_id + course_children_ids
            else:
                break
        return Course.objects.filter(id__in=courses_id)

    def parents_deep(self, course=None, course_id=None):
        course_obj = course
        if course_id:
            try:
                course_obj = Course.objects.get(id=course_id)
            except:
                return Course.objects.none()
        parent = [course_obj.id]
        while course_obj.parent_obj:
            course_obj = course_obj.parent_obj
            parent.append(course_obj.id)

        return Course.objects.filter(id__in=parent)


# Course Model
class Course(BaseCourse):
    MONTH_FARVARDIN = '1'
    MONTH_ORDIBEHESHT = '2'
    MONTH_KHORDAD = '3'
    MONTH_TIR = '4'
    MONTH_MORDAD = '5'
    MONTH_SHAHRIVAR = '6'
    MONTH_MEHR = '7'
    MONTH_ABAN = '8'
    MONTH_AZAR = '9'
    MONTH_DEY = '10'
    MONTH_BAHMAN = '11'
    MONTH_ESFAND = '12'

    MONTH_CHOICES = (
        (MONTH_FARVARDIN, _('Farvardin')),
        (MONTH_ORDIBEHESHT, _('Ordibehesht')),
        (MONTH_KHORDAD, _('Khordad')),
        (MONTH_TIR, _('Tir')),
        (MONTH_MORDAD, _('Mordad')),
        (MONTH_SHAHRIVAR, _('Shahrivar')),
        (MONTH_MEHR, _('Mehr')),
        (MONTH_ABAN, _('Aban')),
        (MONTH_AZAR, _('Azar')),
        (MONTH_DEY, _('Dey')),
        (MONTH_BAHMAN, _('Bahman')),
        (MONTH_ESFAND, _('Esfand')),

    )

    WEEK_FIRST = '1'
    WEEK_SECOND = '2'
    WEEK_THIRD = '3'
    WEEK_FOURTH = '4'
    WEEK_CHOICES = (
        (WEEK_FIRST, _('1')),
        (WEEK_SECOND, _('2')),
        (WEEK_THIRD, _('3')),
        (WEEK_FOURTH, _('4')),
    )

    DAY_FIRST = '1'
    DAY_SECOND = '2'
    DAY_THIRD = '3'
    DAY_FOURTH = '4'
    DAY_FIFTH = '5'
    DAY_SIXTH = '6'
    DAY_SEVENTH = '7'
    DAY_CHOICES = (
        (DAY_FIRST, _('Saturday')),
        (DAY_SECOND, _('Sunday')),
        (DAY_THIRD, _('Monday')),
        (DAY_FOURTH, _('Tuesday')),
        (DAY_FIFTH, _('Wednesday')),
        (DAY_SIXTH, _('Thursday')),
        (DAY_SEVENTH, _('Friday')),
    )
    
    cover = models.ForeignKey(FileField, related_name='course_cover', on_delete=models.CASCADE, verbose_name=_('Cover Image'))
    banner = models.ForeignKey(FileField, related_name='course_banner', on_delete=models.CASCADE, verbose_name=_('Banner Image'))
    # TODO: set category marboot be course
    category_obj = models.ForeignKey(Category, null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('Category'))
    parent_obj = models.ForeignKey('self', null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('Parent'))
    dependency_obj = models.ForeignKey('self', related_name='dependency', null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('Dependency'))
    publish_date = models.DateTimeField(default=now, verbose_name=_('Publish Date'))
    publish_month = models.CharField(max_length=2, null=True, blank=True, choices=MONTH_CHOICES, verbose_name=_('Month'))
    publish_week = models.CharField(max_length=1, null=True, blank=True, choices=WEEK_CHOICES, verbose_name=_('Week'))
    publish_day = models.CharField(max_length=1, null=True, blank=True, choices=DAY_CHOICES, verbose_name=_('DAY'))
    teacher_obj = models.ManyToManyField(Teacher, blank=True, verbose_name=_('Teachers'))

    objects = CourseManager()

    def __str__(self):
        return self.title

    def __init__(self, *args, **kwargs):
        super(Course, self).__init__(*args, **kwargs)

    def clean(self):
        if self.parent_obj:
            course = self
            while course:
                if course.parent_obj:
                    if course.parent_obj.id == self.id:
                        raise ValidationError({'parent_obj': [_('Can not be the parent itself.')]})
                course = course.parent_obj

            value_depth = 2
            try:
                value_depth = Setting.objects.get(key='COURSE_LEVEL').get_value()
            except:
                pass

            if self.depth > value_depth:
                raise ValidationError({'parent_obj': ValidationError(_('Depth must be less than %(value)s'),
                                                                     params={
                                                                         'value': value_depth
                                                                     })
                                       })

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Course, self).save(*args, **kwargs)

    @property
    def depth(self):
        counter = 0
        course = self
        while course.parent_obj:
            counter += 1
            course = course.parent_obj

        return counter

    @property
    def teachers(self):
        teachers = None
        course = self
        while not course.teacher_obj or course.teacher_obj.count() == 0:
            last_course = course.parent_obj
            if not last_course:
                break
            course = last_course

        if course.teacher_obj:
            teachers = course.teacher_obj

        return teachers

    # def get_api_order_uri(self):
    #     return reverse("api:course:step:order", args=[self.course_obj.id, self.id])
    #
    def get_api_file_uri(self):
        return reverse('aparnik-api:educations:courses:files:list', args=[self.id])

    def get_api_file_create_uri(self):
        return reverse('aparnik-api:educations:courses:files:create', args=[self.id])

    # def get_order(self, user):
    #     from order.models import Order
    #     return Order.objects.get_order_product(user=user, product=self)

    def get_api_child_uri(self):
        return reverse('aparnik-api:educations:courses:detail', args=[self.id])

    def get_api_children_id_uri(self):
        return reverse('aparnik-api:educations:courses:list-ids', args=[self.id])

    def get_api_files_id_uri(self):
        return reverse('aparnik-api:educations:courses:files:list-ids', args=[self.id])

    class Meta:
        verbose_name = _('Course')
        verbose_name_plural = _('Courses')

    def is_buy(self, user):

        is_buy = super(Course, self).is_buy(user=user)

        if is_buy:
            return True

        if self.parent_obj:
            return self.parent_obj.is_buy(user=user)

        return False

    def is_user_invited(self, user):
        is_user_invited = super(Course, self).is_user_invited(user=user)

        if is_user_invited:
            return True

        if self.parent_obj:
            return self.parent_obj.is_user_invited(user=user)

        return False

    @property
    def is_free(self):
        is_free = super(Course, self).is_free

        if is_free:
            return True

        if self.parent_obj:
            return self.parent_obj.is_free

        return False

    @property
    def is_subscription(self):

        is_subscription = super(Course, self).is_subscription

        if is_subscription:
            return True

        if self.parent_obj:
            return self.parent_obj.is_subscription

        return False

    def get_user_for_base_review_notification(self, instance, created):
        users = super(Course, self).get_user_for_base_review_notification(instance, created)
        if created:
            return users | User.objects.active().filter(pk__in=self.teachers.active().values_list('user_obj', flat=True))
        return users

    def has_permit_edit(self, user):
        permit = super(Course, self).has_permit_edit(user)
        if permit:
            return True
        return self.teachers.active().filter(user_obj=user).exists()


# File Manager
class CourseFileManager(FileManager):

    def get_queryset(self):
        return super(CourseFileManager, self).get_queryset().order_by('sort')
    
    def active(self, user=None):
        return super(CourseFileManager, self).active(user)

    def get_this_user(self, user):
        # TODO: return this user's course
        # return self.get_queryset().filter(users=user)
        return self.get_queryset().all()

    def get_total_time(self):
        return self.get_queryset().filter(is_free_field=False)

    def file_count(self, course_obj):
        return self.active().filter(course_obj=course_obj).count()


# File Model
class CourseFile(File):

    course_obj = models.ForeignKey(Course, on_delete=models.CASCADE, verbose_name=_('Course'))
    # time = models.TimeField(verbose_name=_('Time'))
    seconds = models.BigIntegerField(default=0, verbose_name=_('Time'))

    objects = CourseFileManager()

    def __init__(self, *args, **kwargs):
        super(CourseFile, self).__init__(*args, **kwargs)

    def __str__(self):
        # self.file.size
        return "%s" % (self.title)

    class Meta:
        verbose_name = _('File')
        verbose_name_plural = _('Files')

    def save(self, *args, **kwargs):
        self.full_clean()
        # self.seconds = datetime.timedelta(hours=self.time.hour, minutes=self.time.minute, seconds=self.time.second).total_seconds()
        if self.file_obj:
            self.seconds = self.file_obj.seconds
        return super(CourseFile, self).save(*args, **kwargs)

    @property
    def is_free(self):

        is_free = super(CourseFile, self).is_free

        if is_free:
            return is_free

        if self.course_obj:
            return self.course_obj.is_free

        return False

    @property
    def is_subscription(self):
        is_subscription = super(CourseFile, self).is_subscription

        if is_subscription:
            return is_subscription

        if self.course_obj:
            return self.course_obj.is_subscription

        return False

    def is_buy(self, user):

        is_buy = super(CourseFile, self).is_buy(user=user)

        if is_buy:
            return is_buy

        if self.course_obj:
            return self.course_obj.is_buy(user=user)

        return False

    def is_user_invited(self, user):
        is_user_invited = super(CourseFile, self).is_user_invited(user=user)

        if is_user_invited:
            return is_user_invited

        if self.course_obj:
            return self.course_obj.is_user_invited(user=user)

        return False

    def get_user_for_base_review_notification(self, instance, created):
        users = super(CourseFile, self).get_user_for_base_review_notification(instance, created)
        if created:
            return users | User.objects.active().filter(pk__in=self.course_obj.teachers.active().values_list('user_obj', flat=True))
        return users

    def has_permit_edit(self, user):
        permit = super(CourseFile, self).has_permit_edit(user)
        if permit:
            return True
        return self.course_obj.teachers.active().filter(user_obj=user).exists()


def post_save_course_receiver(sender, instance, created, *args, **kwargs):
    if instance.sort == 0:
        max = Course.objects.filter(parent_obj=instance.parent_obj).aggregate(Max('sort'))['sort__max']
        instance.sort = max + 1
        instance.save()
    else:
        for course in Course.objects.exclude(id=instance.id).filter(parent_obj=instance.parent_obj, sort=instance.sort):
            course.sort = course.sort + 1
            course.save()


def post_save_file_receiver(sender, instance, created, *args, **kwargs):
    if instance.sort == 0:
        max = CourseFile.objects.filter(course_obj=instance.course_obj).aggregate(Max('sort'))['sort__max']
        instance.sort = max + 1
        instance.save()
    else:
        for file in CourseFile.objects.exclude(id=instance.id).filter(course_obj=instance.course_obj, sort=instance.sort):
            file.sort = file.sort + 1
            file.save()


post_save.connect(post_save_course_receiver, sender=Course)
post_save.connect(post_save_file_receiver, sender=CourseFile)


# Course Segment Model
class CourseSegment(BaseSegment):

    objects = BaseSegmentManager()

    def __str__(self):
        return super(CourseSegment, self).__str__()

    class Meta:
        verbose_name = _('Course Segment')
        verbose_name_plural = _('Courses Segments')


# CourseFile Segment Model
class CourseFileSegment(BaseSegment):

    objects = BaseSegmentManager()

    def __str__(self):
        return super(CourseFileSegment, self).__str__()

    class Meta:
        verbose_name = _('Course File Segment')
        verbose_name_plural = _('Course Files Segments')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-01 17:10


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0017_remove_basecourse_content'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='coursefile',
            name='time',
        ),
    ]



import django
from django.utils.translation import gettext_lazy as _
import rest_framework.authtoken.models
import rest_framework.exceptions
from rest_framework_jwt.serializers import VerifyJSONWebTokenSerializer
from urllib import parse
from channels.auth import AuthMiddlewareStack


class TokenAuthMiddleware:
    """Token authorization middleware for Channels.

    Authenticate the connection by the header 'Authorization: Token...'
    using Django REST framework token-based authentication.
    """

    def __init__(self, inner):
        """Save given inner middleware to invoke in the `__call__`."""
        self._inner = inner

    def __call__(self, scope):
        """Add user to the scope by 'Authorization: Token...' header."""

        # This function carefully and creatively copied from Django REST
        # framework implementation `TokenAuthentication` class.

        # Only handle "Authorization" headers starting with "Token".
        headers = dict(scope["headers"])
        if b"authorization" not in headers:
            # check in get parameters
            query_string = parse.parse_qs(scope['query_string'].decode("utf-8", "replace"))
            if 'token' in query_string and query_string['token'][0]:
                split_token = query_string['token'][0].split()
                # TODO: handle prefix
                if not split_token or len(split_token) != 2:
                    return self._inner(scope)
                data = {'token': split_token[1]}
                valid_data = VerifyJSONWebTokenSerializer().validate(data)
                user = valid_data['user']
                # Call inner middleware with a user in the scope.
                return self._inner(dict(scope, user=user))

            return self._inner(scope)

        # auth_header = headers[b"authorization"].split()
        # if not auth_header or auth_header[0].lower() != "token".encode():
        #     return self._inner(scope)
        #
        # # Check header correctness. Since we use Django REST framework
        # # for token-based authentication, we raise its exceptions.
        # AuthError = rest_framework.exceptions.AuthenticationFailed
        # if len(auth_header) == 1:
        #     raise AuthError(_("Invalid token header: no credentials provided!"))
        # if len(auth_header) > 2:
        #     raise AuthError(_("Invalid token header: token string contains spaces!"))
        # try:
        #     auth_header_token = auth_header[1].decode()
        # except UnicodeError:
        #     raise AuthError(_("Invalid token header: token contains invalid symbols!"))
        #
        # # According to the warning in the Channels authentication docs
        # # we have to manually close old database connections to prevent
        # # usage of timed out connections.
        # django.db.close_old_connections()
        #
        # # Find a user by the token.
        # Token = rest_framework.authtoken.models.Token
        # try:
        #     token = Token.objects.select_related("user").get(key=auth_header_token)
        # except Token.DoesNotExist:
        #     raise AuthError(_("Invalid token!"))
        # if not token.user.is_active:
        #     raise AuthError(_("User is inactive!"))


        # my code
        auth_header = headers[b"authorization"].split()
        # TODO: handle prefix
        if not auth_header or len(auth_header) != 2:
            return self._inner(scope)

        data = {'token': auth_header[1]}
        valid_data = VerifyJSONWebTokenSerializer().validate(data)
        user = valid_data['user']

        # Call inner middleware with a user in the scope.
        return self._inner(dict(scope, user=user))

TokenAuthMiddlewareStack = lambda inner: TokenAuthMiddleware(AuthMiddlewareStack(inner))



"""Celery tasks."""

from __future__ import absolute_import, unicode_literals
import importlib

from celery import shared_task

from . import default_settings as settings


def _import_channel(channel_alias):
    """
    helper to import channels aliases from string paths.

    raises an AttributeError if a channel can't be found by it's alias
    """
    try:
        channel_path = settings.NOTIFICATIONS_CHANNELS[channel_alias]
    except KeyError:
        raise AttributeError(
            '"%s" is not a valid delivery channel alias. '
            'Check your applications settings for NOTIFICATIONS_CHANNELS'
            % channel_alias
        )
    package, attr = channel_path.rsplit('.', 1)

    return getattr(importlib.import_module(package), attr)


@shared_task
def send_notification(notification):
    """Send notification via a channel to celery."""
    # For test
    # import time
    # for i in range(100):
    #     time.sleep(0.1)
    #     if i % 10==0:
    #         print('result:{}'.format(i))
    for channel_alias in notification['channels']:
        channel = _import_channel(channel_alias)(**notification)

        message = channel.construct_message()
        channel.notify(message)



from django import template
from django.core.validators import URLValidator

from aparnik.settings import Setting

register = template.Library()


@register.simple_tag()
def site_name_persian():
    title = Setting.objects.get(key='PROJECT_NAME').get_value()
    return title


@register.simple_tag()
def project_description():
    description = Setting.objects.get(key='PROJECT_DESCRIPTION').get_value()
    return description


@register.simple_tag()
def logo():
    logo = Setting.objects.get(key='LOGO_PROJECT_ICON').get_value()
    return logo


@register.simple_tag(takes_context=True)
def applications_download_list(context):
    def get_script(is_iran, link):
        if not is_iran:
            return '''
            alert("دسترسی تنها از ایران مقدور است. لطفا وی پی ان خود را خاموش بفرمایید.");
            '''
        else:
            return '''window.open("''' + link + '''", "myWin", "scrollbars=yes,width=400,height=650");'''
    #
    # import geoip2.database
    # reader = geoip2.database.Reader('/usr/share/GeoIP/GeoLite2-Country.mmdb')
    # response = reader.country(context['request'].META.get('REMOTE_ADDR'))
    # is_iran = response.country.iso_code.upper() == 'IR'
    is_iran = True

    APPSTORE_LINK = [
        'ios',
        '''window.open("''' + Setting.objects.get(key='APPSTORE_LINK').get_value() + '''", "myWin",
                            "scrollbars=yes,width=400,height=650");self.close()''',
        Setting.objects.get(key='APPSTORE_LINK'),

    ]
    IOS_LINK = [
        'ios',
        get_script(is_iran, Setting.objects.get(key='IOS_LINK').get_value()),
        Setting.objects.get(key='IOS_LINK')
    ]
    GOOGLE_PLAY_LINK = [
        'googleplay',
        '''window.open("''' + Setting.objects.get(key='GOOGLE_PLAY_LINK').get_value() + '''", "myWin",
                    "scrollbars=yes,width=400,height=650");''',
        Setting.objects.get(key='GOOGLE_PLAY_LINK')
    ]
    ANDROID_LINK = [
        'android',
        '''window.open("''' + Setting.objects.get(key='ANDROID_LINK').get_value() + '''", "myWin",
                        "scrollbars=yes,width=400,height=650");''',
        Setting.objects.get(key='ANDROID_LINK')
    ]

    validator = URLValidator()

    apdl = []
    for value in [APPSTORE_LINK, IOS_LINK, GOOGLE_PLAY_LINK, ANDROID_LINK]:
        try:
            # check validation url if not valid raise it happen
            validator(value[2].get_value())
            if value[2].get_value() != '':
                apdl.append({
                    'class': value[0],
                    'js': value[1]
                })
        except:
            pass

    return apdl


@register.simple_tag()
def application_image():
    return Setting.objects.get(key='APPLICATION_IMAGE').get_value()


@register.simple_tag()
def application_identifier():
    return Setting.objects.get(key='IDENTIFIER').get_value()



from django.conf.urls import url, include

from .views import QAListAPIView, QADetailAPIView, QACreateAPIView, QASortAPIView

app_name = 'questionanswers'

urlpatterns = [
    url(r'^$', QAListAPIView.as_view(), name='list'),
    url(r'^create/$', QACreateAPIView.as_view(), name='create'),
    url(r'^sort/$', QASortAPIView.as_view(), name='sort'),
    url(r'^model/(?P<model_id>\d+)/$', QAListAPIView.as_view(), name='models-list'),
    url(r'^(?P<qa_id>\d+)/$', QADetailAPIView.as_view(), name='detail'),
]


"""Celery tasks."""

from __future__ import absolute_import, unicode_literals

import time

from celery import shared_task

from aparnik.contrib.users.models import User
from aparnik.packages.shops.orders.api.serializers import OrderSummarySerializer
from aparnik.packages.shops.orders.models import *
from aparnik.contrib.messaging.utils import notify
from aparnik.utils.utils import get_request


@shared_task
def send_order_message(order_id, is_create):
    """Send chat message create notification and etc via a channel to celery."""
    time.sleep(0.2)
    order_obj = Order.objects.get(pk=order_id)
    serializer = OrderSummarySerializer(order_obj, many=False, read_only=True, context={'request': get_request()})
    users = User.objects.admins()
    for user in users:
        notify(
            uri=order_obj.user.username,
            source=order_obj.user,
            source_display_name=order_obj.user.get_full_name(),
            recipient=user,
            action='Create' if is_create else 'Modified',
            obj=order_obj,
            short_description='You have a {} order.'.format('new' if is_create else 'modified'),
            extra_data={
                'uri': order_obj.user.username,
                'message': serializer.data,
            },
            channels=['push'],
            silent=True,
        )



# Generated by Django 2.2 on 2020-01-05 15:26

import aparnik.contrib.chats.models
from django.db import migrations
import django_enumfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('chats', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='chatsession',
            name='type',
            field=django_enumfield.db.fields.EnumField(default=1, enum=aparnik.contrib.chats.models.ChatSessionTypeEnum, verbose_name='Type'),
        ),
    ]



from django.conf import settings
from rest_framework.settings import APISettings

USER_SETTINGS = getattr(settings, 'APARNIK', None)

DEFAULTS = {
    'API_PRODUCT_MODE': False,
    'AWS_ACTIVE': True,
    'BANK_ACTIVE': True,
    'ZARINPAL_MERCHENT_CODE': '',
    'USER_LOGIN_WITH_PASSWORD': False,
    'USER_CREATE_SERIALIZER': 'aparnik.contrib.users.api.serializers.UserCreateSerializer',
    'USER_UPDATE_SERIALIZER': 'aparnik.contrib.users.api.serializers.UserUpdateSerializer',
    'USER_DETAIL_SERIALIZER': 'aparnik.contrib.users.api.serializers.UserDetailSerializer',
    'USER_SUMMARY_LIST_SERIALIZER': 'aparnik.contrib.users.api.serializers.UserSummaryListSerializer',
    'USER_LOGIN_API_VIEW': 'aparnik.contrib.users.api.views.UserLoginAPIView',
    'USER_LIST_API_VIEW': 'aparnik.contrib.users.api.views.UserListAPIView',
    'USER_DETAIL_API_VIEW': 'aparnik.contrib.users.api.views.UserDetailAPIView',
    'USER_CREATE_API_VIEW': 'aparnik.contrib.users.api.views.UserCreateAPIView',
    'USER_UPDATE_API_VIEW': 'aparnik.contrib.users.api.views.UserUpdateAPIView',
    'USER_INVITATION_CODE_LENGTH': 6,
    'USER_INVITATION_CODE_INT': False,
    'USER_IS_SHOW_ADMIN_PANEL': True,
    'USER_REGISTER_WITH_LOGIN': False,
    'SMS_EXPIRE_TOKEN': 120,
    'SMS_API_KEY': '',
    'SMS_OTA_NAME': '',
    'SEND_SMS': False,
    'VERIFY_FAKE': False,
    'VERIFY_FAKE_CODE': '',
    'JALALI_FORMAT': '%Y/%m/%d',
    'INVITATION_PERCENT_DISCOUNT': 0,
    'COUPON_CODE_LENGTH': 5,
    'IS_FCM': True,
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = (
    'AWS_ACTIVE',
    'BANK_ACTIVE',
    'USER_LOGIN_WITH_PASSWORD',
    'USER_CREATE_SERIALIZER',
    'USER_UPDATE_SERIALIZER',
    'USER_DETAIL_SERIALIZER',
    'USER_SUMMARY_LIST_SERIALIZER',
    'USER_LOGIN_API_VIEW',
    'USER_LIST_API_VIEW',
    'USER_DETAIL_API_VIEW',
    'USER_CREATE_API_VIEW',
    'USER_UPDATE_API_VIEW',
    'USER_INVITATION_CODE_INT',
    'USER_IS_SHOW_ADMIN_PANEL',
    'USER_REGISTER_WITH_LOGIN',
    'API_PRODUCT_MODE',
    'SEND_SMS',
    'VERIFY_FAKE',
    'IS_FCM',
)

aparnik_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)
from aparnik.contrib.settings.models import Setting



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-03 22:33


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('progresses', '0002_progresssummary'),
    ]

    operations = [
        migrations.AlterField(
            model_name='progresses',
            name='file_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='courses.CourseFile', verbose_name='File'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-25 19:37


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filefields', '0004_filefield_title'),
        ('basemodels', '0004_auto_20181103_2233'),
        ('segments', '0002_auto_20181026_1745'),
    ]

    operations = [
        migrations.CreateModel(
            name='Button',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('title', models.CharField(blank=True, max_length=255, null=True, verbose_name='\\u0639\\u0646\\u0648\\u0627\\u0646')),
                ('icon', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='\\u0622\\u06cc\\u06a9\\u0646')),
                ('model_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='button_model', to='basemodels.BaseModel', verbose_name='\\u0686\\u0647 \\u0686\\u06cc\\u0632 \\u0631\\u0627 \\u0646\\u0645\\u0627\\u06cc\\u0634 \\u062f\\u0647\\u062f\\u061f')),
            ],
            options={
                'verbose_name': 'Button',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Buttons',
            },
            bases=('basemodels.basemodel',),
        ),
        migrations.CreateModel(
            name='ButtonSegment',
            fields=[
                ('basesegment_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='segments.BaseSegment')),
            ],
            options={
                'verbose_name': 'Button Segment',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Button Segments',
            },
            bases=('segments.basesegment',),
        ),
    ]



# -*- coding: utf-8 -*-


from django.conf.urls import url
from . import views

app_name = 'bmi'

urlpatterns = [
    url(r'^request/(?P<id>[0-9]+)/', views.send_request, name='request'),
    url(r'^verify/(?P<uuid>[0-9a-f-]+)/$', views.verify, name='verify'),
]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path. abspath(__file__), os.pardir)))

setup(
    name='django-apar',
    version='1.1.6.36',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',  # example license
    description='Aparnik is a simple Django app to help about some common problems.',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://www.aparnik.com/',
    author='Ali Zahedi Gol',
    author_email='zahedi@aparnik.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'boto3',
        'humanize',
        'django==2.2',
        'django-polymorphic',
        'django-rest-polymorphic',
        'pycryptodome',
        'django-s3direct',
        'djangorestframework',
        'python-dateutil',
        'kavenegar',
        'django-jalali-date',
        'Pillow',
        'fcm-django',
        'zeep',
        'django-dynamic-raw-id',
        'APScheduler',
        'django-filter==2.1.0',
        'django-ckeditor',
        'django-tagulous',
        'django-colorfield',
        'Faker',
        'python-magic',
        'PyPDF2==1.26.0',
        'django-enumfield==1.5',
        'geoip2',
        'redis==3.2.0',
        'channels==2.4.0',
        'channels-redis==2.4.1',
        'celery==4.4.0',
        'django-admin-rangefilter==0.5.3',
        'django-import-export==2.0.1',
        'uvicorn==0.11.1'
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-17 16:51


import aparnik.contrib.notifications.models
from django.db import migrations
import django_enumfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0003_auto_20190517_1651'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='notification',
            name='is_for_all_users',
        ),
        migrations.AlterField(
            model_name='notification',
            name='notification_send_type',
            field=django_enumfield.db.fields.EnumField(default=2, enum=aparnik.contrib.notifications.models.NotificationType, verbose_name='Notification send type'),
        ),
    ]



"""Utilities and helper functions."""
from aparnik.contrib.messaging.api.serializers import MessagingListSerializer
from . import MessagingError
from .models import Messaging
from .tasks import send_notification


def notify(
        uri,
        source,
        source_display_name,
        recipient,
        action,
        obj,
        short_description,
        extra_data,
        channels,
        silent=False,
):
    """Helper method to send a notification."""
    notification = Messaging(
        source=source,
        source_display_name=source_display_name,
        recipient=recipient,
        category=obj.resourcetype,
        action=action.upper(),
        obj=obj.id,
        uri=uri,
        short_description=short_description,
        channels=channels,
        extra_data=extra_data,
    )

    # If it's a not a silent notification, save the notification
    if not silent:
        notification.save()

    # Send the notification asynchronously with celery
    send_notification.delay(MessagingListSerializer(notification, many=False, read_only=True, context={''}).data)


def read(notify_id, recipient):
    """
    Helper method to read a notification.

    Raises NotificationError if the user doesn't have access
    to read the notification
    """
    notification = Messaging.objects.get(id=notify_id)

    if recipient != notification.recipient:
        raise MessagingError('You cannot read this notification')

    notification.read()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-26 10:43


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0022_user_company_economical_number'),
    ]

    operations = [
        migrations.RenameField(
            model_name='devicelogin',
            old_name='device_type',
            new_name='device_type_temp',
        ),
    ]



# -*- coding: utf-8 -*-

from django.shortcuts import render, get_object_or_404, redirect, reverse
from django.http.response import JsonResponse
from django.utils.html import strip_tags

from aparnik.contrib.basemodels.models import BaseModel


def install(request):
    context = {
        'title': 'نصب'
    }
    template_name = 'aparnik/index.html'
    return render(request, template_name=template_name, context=context)


def share(request):
    # q['id'] = model.pk
    # q['type'] = model.resourcetype
    # q['ref'] = ''
    # TODO: handle
    # چه کسی فرستاده؟
    # چه کسی باز کرده؟
    pk = request.GET.get('id', 0)
    ref = request.GET.get('ref', '')
    # TODO: use ref
    model = get_object_or_404(BaseModel.objects.all(), pk=pk).get_real_instance()
    title = strip_tags(model.get_title())
    description = strip_tags(model.get_description())

    context = {
        'title': title,
        'description': description,
        'app_url': request.build_absolute_uri(model.get_api_uri()),
        'install_app_url': request.build_absolute_uri(reverse('aparnik:install')),
        'model': model,
    }
    template_name = 'aparnik/share.html'
    return render(request, template_name=template_name, context=context)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-03-19 17:53


import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0019_auto_20190319_1747'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='username_mention',
            field=models.CharField(default='a', max_length=100, unique=True, validators=[django.core.validators.RegexValidator(code='invalid_username', message='Username must be Alphanumeric', regex='^[a-zA-Z_\\-0-9]+$')], verbose_name='Username for mention'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-26 13:22


from django.db import migrations


def rename_admin_groups(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Group = apps.get_model('auth', 'Group')
    try:
        group = Group.objects.get(pk=1)
        group.name = 'admin'
        group.save()

    except Exception:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('managements', '0002_auto_20181105_2023'),
    ]

    operations = [
        migrations.RunPython(rename_admin_groups, reverse_code=migrations.RunPython.noop),
    ]



from rest_framework import serializers

from aparnik.contrib.basemodels.api.admin.serializers import BaseModelAdminSerializer


from aparnik.packages.shops.coupons.models import Coupon


class CouponAdminListSerializer(BaseModelAdminSerializer):


    def __init__(self, *args, **kwargs):
        super(CouponAdminListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Coupon
        fields = BaseModelAdminSerializer.Meta.fields + [
            'code'
        ]

        read_only_fields = BaseModelAdminSerializer.Meta.read_only_fields + [

        ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.contrib.auth.models import Group, Permission
from aparnik.settings import aparnik_settings
from django.utils.translation import ugettext_lazy as _

from copy import deepcopy
from jalali_date.admin import ModelAdminJalaliMixin
from jalali_date import datetime2jalali


from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.contrib.users.models import User, DeviceLogin, UserToken

# Base Function
# foreign key user


def get_first_name(obj):
    return obj.user.first_name


def get_last_name(obj):
    return obj.user.last_name


def get_username(obj):
    return obj.user.username


# Sample usage: get_common_fields('user_obj')+ [rest of fields]
#   => ['user_obj__username', 'user_obj__first_name', 'user_obj__last_name',...]
def get_user_search_fields(field_name):
    field_string = [field_name + '__username', field_name + '__first_name', field_name + '__last_name']
    return field_string


get_last_name.short_description = _('Last Name')
get_username.short_description = _('Mobile')
get_first_name.short_description = _('Name')

# for admin


def get_update_at(obj):
    return datetime2jalali(obj.update_at).strftime(aparnik_settings.JALALI_FORMAT)


get_update_at.admin_order_field = 'update_at'
get_update_at.short_description = 'بروز شده'


def get_last_login(obj):
    if obj.last_login:
        return datetime2jalali(obj.last_login).strftime(aparnik_settings.JALALI_FORMAT)
    return obj.last_login


get_last_login.admin_order_field = 'last_login'
get_last_login.short_description = 'آخرین ورود'


def get_created_at(obj):
    return datetime2jalali(obj.created_at).strftime(aparnik_settings.JALALI_FORMAT)


get_update_at.admin_order_field = 'created_at'
get_update_at.short_description = 'ایجاد'


class UserAdmin(BaseModelAdmin):
# class UserAdmin(admin.ModelAdmin):
# 'is_superuser', 'groups', 'user_permissions', 'avatar',
#     exclude = [, 'token', 'token_time', 'current_city', ]
    list_display_links = ('username', 'first_name', 'last_name')
    # list_editable = ('username', 'mobile', )
    # inlines = [DocumentFilesInline, ]

    fields = []
    list_display = ['username', 'first_name', 'last_name', 'get_device_login_count', 'limit_device_login', get_last_login, get_created_at, 'created_at']
    list_filter = ['is_staff', 'created_at']
    search_fields = ['username', 'last_name', 'first_name']
    exclude = ['last_login', ]
    raw_id_fields = ['current_city', 'avatar']
    dynamic_raw_id_fields = []

    actions = None

    class Meta:
        model = User

    def __init__(self, *args, **kwargs):
        Klass = UserAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_display.remove('id')
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    # permission
    def has_delete_permission(self, request, obj=None):
        # Disable delete
        return False

    def has_add_permission(self, request):
        return True

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = True
        extra_context['show_save'] = True
        extra_context['show_save_and_add_another'] = True  # this not works if has_add_permision is True
        return super(UserAdmin, self).change_view(request, object_id, extra_context=extra_context)

    def get_device_login_count(self, instance):
        return instance.device_login_count()

    get_device_login_count.short_description = 'دستگاه فعال'

    # Removing superuser from staff profile
    def get_fieldsets(self, request, obj=None):
        fieldsets = super(UserAdmin, self).get_fieldsets(request, obj)
        # if not obj:
        #     return fieldsets

        if not request.user.is_superuser:  # or request.user.pk == obj.pk:
            fieldsets = deepcopy(fieldsets)
            for fieldset in fieldsets:
                if 'is_superuser' in fieldset[1]['fields']:
                    if type(fieldset[1]['fields']) == tuple:
                        fieldset[1]['fields'] = list(fieldset[1]['fields'])
                    fieldset[1]['fields'].remove('is_superuser')
                    break
            for fieldset in fieldsets:
                if 'wallet' in fieldset[1]['fields']:
                    if type(fieldset[1]['fields']) == tuple:
                        fieldset[1]['fields'] = list(fieldset[1]['fields'])
                    fieldset[1]['fields'].remove('wallet')
                    break

        return fieldsets

    # remove superusers from staff users list
    def get_queryset(self, request):
        qs = super(UserAdmin, self).get_queryset(request)
        if not request.user.is_superuser:
            return qs.filter(is_superuser=False)
        return qs

    def get_form(self, request, obj=None, **kwargs):
        # Get form from original UserAdmin.
        form = super(UserAdmin, self).get_form(request, obj, **kwargs)
        if 'user_permissions' in form.base_fields:
            user = request.user
            if not user.is_superuser:
                permissions = form.base_fields['user_permissions']
                groups = form.base_fields['groups']
                permissions.queryset = Permission.objects.filter(user=user)
                groups.queryset = Group.objects.filter(user=user)
        return form


class DeviceLoginAdmin(ModelAdminJalaliMixin, admin.ModelAdmin):

    exclude = ('last_login', 'token', 'token_time')
    list_display = ('id', get_username, get_first_name, get_last_name, 'device_id', 'os_version', 'device_model', get_update_at)
    list_display_links = None
    # list_filter = ('is_staff',)
    # list_editable = ('username', 'mobile', )
    search_fields = get_user_search_fields('user') + ['device_id']
    readonly_fields = ( )
    # inlines = [DocumentFilesInline, ]

    actions = None

    # class Meta:
    #     model = User

    # # permission
    # def has_delete_permission(self, request, obj=None):
    #     # Disable delete
    #     return False
    #
    # def has_add_permission(self, request):
    #     return False
    #
    # def change_view(self, request, object_id, form_url='', extra_context=None):
    #     ''' customize edit form '''
    #     extra_context = extra_context or {}
    #     extra_context['show_save_and_continue'] = False
    #     extra_context['show_save'] = False
    #     extra_context['show_save_and_add_another'] = False  # this not works if has_add_permision is True
    #     return super(UserAdmin, self).change_view(request, object_id, extra_context=extra_context)


class UserTokenAdmin(ModelAdminJalaliMixin, admin.ModelAdmin):

    list_display = ('id', 'device_obj', 'user_obj', 'is_active', get_update_at)
    list_filter = ['is_active']
    # list_editable = ('username', 'mobile', )
    search_fields = get_user_search_fields('user_obj') + ['device_obj__device_id']
    readonly_fields = ( )
    raw_id_fields = ['user_obj', 'device_obj']
    # inlines = [DocumentFilesInline, ]

    actions = None


if aparnik_settings.USER_IS_SHOW_ADMIN_PANEL:
    admin.site.register(User, UserAdmin)
    admin.site.register(DeviceLogin, DeviceLoginAdmin)

admin.site.register(UserToken, UserTokenAdmin)
# Now register the new UserAdmin...
# admin.site.register(User, UserAdmin)
# ... and, since we're not using Django's built-in permissions,
# unregister the Group model from admin.
# admin.site.unregister(Group)


# Generated by Django 2.2 on 2020-02-28 10:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0009_auto_20190821_1543'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='type',
            field=models.CharField(choices=[('i', 'اطلاعات'), ('ch', 'Chat'), ('s', 'موفقیت'), ('w', 'هشدار'), ('e', 'خطا')], max_length=2, verbose_name='Type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-03-15 17:17


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bankaccounts', '0002_auto_20190215_1916'),
    ]

    operations = [
        migrations.AddField(
            model_name='bankaccount',
            name='card_number',
            field=models.CharField(default="1111111111111111", max_length=16, verbose_name='Card Number'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from aparnik.packages.shops.payments.models import Payment

# Create your models here.


class BankManager(models.Manager):
    def get_queryset(self):
        return super(BankManager, self).get_queryset()

    def get_this_user(self, user):
        if not user.is_authenticated:
            return Bank.objects.none()
        return self.get_queryset().filter(payment__user=user)

    def get_this_success(self):
        return self.get_queryset().filter(Q(ref_id='-1'))


class Bank(models.Model):

    TRANSACTION_SUCCESS = 100
    TRANSACTION_SUBMITTED = 101
    TRANSACTION_FAILED = 102
    TRANSACTION_FAILED_BY_USER = -1
    TRANSACTION_WAITING = -2

    TRANSACTION_STATUS = (
        (TRANSACTION_SUCCESS, _('Success')),
        (TRANSACTION_SUBMITTED, _('Submitted')),
        (TRANSACTION_FAILED, _('Failed')),
        (TRANSACTION_FAILED_BY_USER, _('Cancel By User')),
        (TRANSACTION_WAITING, _('Waiting for pay')),
    )

    status = models.IntegerField(
        verbose_name=_('Status'),
        # choices=TRANSACTION_STATUS,
        default=-2
    )

    authority_id = models.CharField(
        max_length=255,
        verbose_name=_('Authority ID')
    )

    ref_id = models.CharField(
        null=True,
        blank=True,
        max_length=255,
        verbose_name=_('Bank Reference ID')
    )

    payment = models.ForeignKey(
        Payment,
        on_delete=models.CASCADE,
        verbose_name=_('Payment')
    )

    created_at = models.DateTimeField(
        verbose_name=_('Created At'),
        auto_now_add=True
    )

    update_at = models.DateTimeField(
        verbose_name=_('Update At'),
        auto_now=True
    )

    objects = BankManager()

    def __str__(self):
        return '{}'.format(self.ref_id)

    def __init__(self, *args, **kwargs):
        super(Bank, self).__init__(*args, **kwargs)

    class Meta:
        verbose_name = _('Bank')
        verbose_name_plural = _('Banks')
        ordering = ['created_at']

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Bank, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-


from django.shortcuts import render, get_list_or_404

from aparnik.settings import aparnik_settings
from .models import Payment
from aparnik.packages.bankgateways.utils import send_to_bank

# Create your views here.
def payment(request, uuid):
    payments_obj = get_list_or_404(Payment.objects.all(), uuid=uuid)
    pay_obj = payments_obj[0]

    if pay_obj.order_obj.status == Payment.STATUS_WAITING and pay_obj.status == Payment.STATUS_WAITING:

        if aparnik_settings.BANK_ACTIVE:
            # (bank_obj, created) = BankAPI.objects.get_or_create(amount=pay_obj.transaction.amount, invoice_number=pay_obj.uuid.int)
            return send_to_bank(request, pay_obj)
            # return send_request(request, pay_obj)
        else:
            pay_obj.success()

    context = {
        'obj': pay_obj,
        'app_url': request.build_absolute_uri(pay_obj.get_api_uri()),
    }

    if pay_obj.is_success():
        return render(request=request, template_name='suit/paysuc.html', status=200, context=context)

    return render(request=request, template_name='suit/payunsuc.html', status=400, context=context)




# -*- coding: utf-8 -*-


from django.db import models
from django.db.models.signals import post_save
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

import uuid

from aparnik.settings import Setting
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.packages.shops.orders.models import Order

User = get_user_model()
# Create your models here.


class PaymentManager(BaseModelManager):

    def get_queryset(self):
        return super(PaymentManager, self).get_queryset().order_by('-update_at')

    def get_this_user(self, user):
        if not user.is_authenticated:
            return Payment.objects.none()

        return self.get_queryset().filter(user=user)

    def get_paid(self, user, transaction):
        return self.get_this_user(user=user).filter(transaction=transaction).filter(status=Payment.STATUS_COMPLETE)

    def request_pay(self, order, method, user, call_back_url=""):
        # type: (object, object, object, object) -> object
        # TODO: before pay check order allow or canceled
        return Payment.objects.create(user=user, method=method, status=Payment.STATUS_WAITING, order_obj=order, call_back_url=call_back_url)


class Payment(BaseModel):
    # Method:
    METHOD_BANK = 'b'
    METHOD_WALLET = 'w'
    METHOD_CHOICES = (
        (METHOD_BANK, _('Bank')),
        (METHOD_WALLET, _('Wallet')),
    )
    # Status
    STATUS_CANCEL = 'c'
    STATUS_CANCEL_USER_PAY = 'cup'
    STATUS_CANCEL_SYSTEM = 'cs'
    STATUS_CANCEL_WALLET_DOES_NOT_CHARGE = 'cwn'
    STATUS_COMPLETE = 'co'
    STATUS_WAITING = 'w'
    STATUS_CHOICES = (
        (STATUS_CANCEL, _('Cancel')),
        (STATUS_CANCEL_SYSTEM, _('Cancel by system')),
        (STATUS_CANCEL_SYSTEM, _('Cancel by user after pay begin')),
        (STATUS_COMPLETE, _('Complete')),
        (STATUS_WAITING, _('Waiting')),
    )
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, verbose_name=_('Reference ID'))
    bank_reference = models.CharField(max_length=50, editable=False, blank=True, null=True, verbose_name=_('Bank Reference ID'))
    user = models.ForeignKey(User, editable=False, on_delete=models.CASCADE, verbose_name=_('User'))
    method = models.CharField(max_length=3, editable=False, choices=METHOD_CHOICES, verbose_name=_('Method payment'))
    status = models.CharField(max_length=10, editable=False, choices=STATUS_CHOICES, verbose_name=_('Status'))
    order_obj = models.ForeignKey(Order, editable=False, related_name='payments', on_delete=models.CASCADE, verbose_name=_('Order'))
    call_back_url = models.CharField(max_length=255, editable=False, blank=True, null=True, verbose_name=_('Call Back URL'))

    objects = PaymentManager()

    class Meta:
        verbose_name = _('payment')
        verbose_name_plural = _('Payments')

    def __str__(self):
        return " %s - %s" %(self.user, self.method)

    def clean(self):
        # Don't allow complete action entries.
        if self.id:
            obj = Payment.objects.get(id=self.id)
        else:
            obj = self
        if obj.status != Payment.STATUS_WAITING or obj.order_obj.status != Payment.STATUS_WAITING:
            raise ValidationError(_('payment finish before.'))

        if obj.status == Payment.STATUS_WAITING and obj.method == Payment.METHOD_BANK and not obj.call_back_url:
            raise ValidationError(_('payment for bank must fill call back url.'))

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Payment, self).save(*args, **kwargs)

    def get_url(self):
        if self.method == Payment.METHOD_WALLET:
            return reverse('aparnik-api:shops:payments:payment', args=[self.uuid])
        return reverse('aparnik:shops:payments:payment', args=[self.uuid]) #kwargs={'app_label': 'auth'}

    def get_method_json_display(self):
        icon = ""
        if self.method == Payment.METHOD_BANK:
            icon = "http://www.freeiconspng.com/uploads/bank-account-integration-with-existing-bank-accounts-for-inclusion--22.png"
        else:
            icon = 'http://www.freeiconspng.com/uploads/wallet-icon-7.png'
        json = {
            # TODO: change icon
            'icon': icon,
            'name': self.get_method_display()
        }
        return json

    def get_api_pay_cancel_uri(self):
        return reverse('aparnik-api:shops:payments:cancel', args=[self.uuid])

    def get_call_back_url(self):
        status = 0
        if self.status == Payment.STATUS_COMPLETE:
            status = 1

        import urllib.request, urllib.parse, urllib.error
        paramters = {
            'status': status,
            'date': self.created_at,
            'price': self.order_obj.total_cost_string.encode('utf8'),
            'refrence-id': self.uuid,
            'title': self.order_obj.items.first().product_obj.title.encode('utf8')
        }

        return self.call_back_url + '?' + urllib.parse.urlencode(paramters)

    def success(self):
        from aparnik.packages.shops.vouchers.models import Voucher
        if self.order_obj.is_allow_to_pay():
            self.status = Payment.STATUS_COMPLETE
            if self.method == self.METHOD_WALLET:
                self.user.wallet = self.user.wallet - self.order_obj.get_total_cost()
                if self.user.wallet < 0:
                    self.status = Payment.STATUS_CANCEL_WALLET_DOES_NOT_CHARGE
                    self.save()
                    charge_wallet_message = Setting.objects.get(key='WALLET_CHARGING_MESSAGE').get_value()
                    raise ValidationError(
                        # {'order': [_('Your wallet does not charge. Please charge it first.')]})
                        {'order': [charge_wallet_message]})

                Voucher.objects.redeem_for_order(self.order_obj)
                self.user.save()
            else:
                Voucher.objects.redeem_for_order(self.order_obj)

        else:
            self.status = Payment.STATUS_CANCEL_SYSTEM
        self.save()

        if self.status == Payment.STATUS_CANCEL_SYSTEM:
            raise ValidationError({'order': [_('You\'re order has been canceled by system. Please contact with shopkeeper.')]})

    def cancel(self):
        self.status = Payment.STATUS_CANCEL
        self.save()

    def is_success(self):
        return self.status == Payment.STATUS_COMPLETE


def post_save_payment_receiver(sender, instance, created, *args, **kwargs):

    if instance.status == Payment.STATUS_COMPLETE:
        from aparnik.packages.shops.products.models import Product
        wallet_id = Product.objects.get_wallet().id
        instance.order_obj.pay_success()

        try:
            sync_api_address_url = Setting.objects.active().get(key='ORDER_SYNC_WITH_WEBSITES').get_value()
            secret_key = Setting.objects.active().get(key='SECRET_KEY').get_value()
            import requests
            params = {'secret_key': secret_key, 'username': instance.order_obj.user.username,
                                    'products_id': ",".join(
                                        str(item.product_obj.pk) for item in instance.order_obj.items.all())}
            r = requests.post(sync_api_address_url,
                              data=params)

            if r.status_code == 200:
                instance.order_obj.is_sync_with_websites = True
                instance.order_obj.save()
        except:
            pass

        # check invite and etc.
        for item in instance.order_obj.items.all():

            if item.product_obj.id == wallet_id:
                return

        # فرد دعوت شده است یا خیر
        if instance.order_obj.user.invite.first():
            from aparnik.packages.shops.cosales.models import CoSale
            # کسی که دعوت کرده
            invited_by = instance.order_obj.user.invite.first().invited_by
            CoSale.objects.add_record(user_bought=instance.order_obj.user, user_co_sale=invited_by, order=instance.order_obj)

    elif instance.status == Payment.STATUS_CANCEL_SYSTEM:
        instance.order_obj.cancel()


post_save.connect(post_save_payment_receiver, sender=Payment)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-02 15:42


import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0009_auto_20181130_2025'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='product',
            name='has_permit_use_aparnik_bon_value',
        ),
        migrations.AddField(
            model_name='product',
            name='has_permit_use_wallet_value',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply', validators=[django.core.validators.MaxValueValidator(1), django.core.validators.MinValueValidator(-1)], verbose_name='Has Permit use wallet'),
        ),
        migrations.AddField(
            model_name='product',
            name='maximum_use_aparnik_bon_value',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply', verbose_name='Maximum use aparnik bon'),
        ),
        migrations.AlterField(
            model_name='product',
            name='aparnik_bon_return_expire_value',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply', verbose_name='Bon Return Expire ( Hours )'),
        ),
        migrations.AlterField(
            model_name='product',
            name='aparnik_bon_return_value',
            field=models.IntegerField(default=-1, help_text='If the number is set to -1, then the value in the settings will apply', verbose_name='Bon Return'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bookmarks', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='bookmark',
            options={'verbose_name': '\\u0628\\u0648\\u06a9\\u0645\\u0627\\u0631\\u06a9', 'verbose_name_plural': '\\u0628\\u0648\\u06a9\\u0645\\u0627\\u0631\\u06a9 \\u0647\\u0627'},
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='\\u0633\\u0627\\u062e\\u062a\\u0647 \\u0634\\u062f\\u0647 \\u062f\\u0631'),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='model_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='bookmark_obj', to='basemodels.BaseModel', verbose_name='\\u0686\\u0647 \\u0686\\u06cc\\u0632 \\u0631\\u0627 \\u0646\\u0645\\u0627\\u06cc\\u0634 \\u062f\\u0647\\u062f\\u061f'),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='update_at',
            field=models.DateTimeField(auto_now=True, verbose_name='\\u0628\\u0631\\u0648\\u0632\\u0631\\u0633\\u0627\\u0646\\u06cc \\u0634\\u062f\\u0647 \\u062f\\u0631'),
        ),
    ]
